@@ -1,4 +1,4 @@
-// This file Copyright © 2007-2023 Mnemosyne LLC.
+// This file Copyright © Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
@@ -14,7 +14,6 @@
 #include "Utils.h"
 
 #include <libtransmission/transmission.h>
-#include <libtransmission/version.h>
 #include <libtransmission/web-utils.h>
 
 #include <glibmm/date.h>
@@ -41,13 +40,18 @@
 #include <gtkmm/eventcontrollerfocus.h>
 #endif
 
-#include <fmt/core.h>
+#include <fmt/format.h>
 
+#include <array>
 #include <limits>
 #include <map>
 #include <memory>
+#include <optional>
 #include <sstream>
 #include <string>
+#include <string_view>
+
+using namespace libtransmission::Values;
 
 /**
 ***
@@ -57,10 +61,12 @@ class PrefsDialog::Impl
 {
 public:
     Impl(PrefsDialog& dialog, Glib::RefPtr<Gtk::Builder> const& builder, Glib::RefPtr<Session> const& core);
+    Impl(Impl&&) = delete;
+    Impl(Impl const&) = delete;
+    Impl& operator=(Impl&&) = delete;
+    Impl& operator=(Impl const&) = delete;
     ~Impl() = default;
 
-    TR_DISABLE_COPY_MOVE(Impl)
-
 private:
     void response_cb(int response);
 
@@ -93,10 +99,12 @@ class PageBase : public Gtk::Box
 {
 public:
     PageBase(BaseObjectType* cast_item, Glib::RefPtr<Gtk::Builder> const& builder, Glib::RefPtr<Session> const& core);
+    PageBase(PageBase&&) = delete;
+    PageBase(PageBase const&) = delete;
+    PageBase& operator=(PageBase&&) = delete;
+    PageBase& operator=(PageBase const&) = delete;
     ~PageBase() override;
 
-    TR_DISABLE_COPY_MOVE(PageBase)
-
     Gtk::CheckButton* init_check_button(Glib::ustring const& name, tr_quark key);
     Gtk::SpinButton* init_spin_button(Glib::ustring const& name, tr_quark key, int low, int high, int step);
     Gtk::SpinButton* init_spin_button_double(Glib::ustring const& name, tr_quark key, double low, double high, double step);
@@ -122,7 +130,7 @@ public:
     template<typename T, typename... ArgTs>
     static void localize_label(T& widget, ArgTs&&... args)
     {
-        widget.set_label(fmt::format(widget.get_label().raw(), std::forward<ArgTs>(args)...));
+        widget.set_label(fmt::format(fmt::runtime(widget.get_label().raw()), std::forward<ArgTs>(args)...));
     }
 
 private:
@@ -307,7 +315,8 @@ Gtk::ComboBox* PageBase::init_encryption_combo(Glib::ustring const& name, tr_qua
             { _("Prefer encryption"), TR_ENCRYPTION_PREFERRED },
             { _("Require encryption"), TR_ENCRYPTION_REQUIRED },
         });
-    gtr_combo_box_set_active_enum(*combo, gtr_pref_int_get(key));
+    auto const mode = gtr_pref_get<tr_encryption_mode>(key).value_or(TR_ENCRYPTION_PREFERRED);
+    gtr_combo_box_set_active_enum(*combo, static_cast<int>(mode));
     combo->signal_changed().connect([this, combo, key]() { onIntComboChanged(*combo, key); });
     return combo;
 }
@@ -395,10 +404,12 @@ class DownloadingPage : public PageBase
 {
 public:
     DownloadingPage(BaseObjectType* cast_item, Glib::RefPtr<Gtk::Builder> const& builder, Glib::RefPtr<Session> const& core);
+    DownloadingPage(DownloadingPage&&) = delete;
+    DownloadingPage(DownloadingPage const&) = delete;
+    DownloadingPage& operator=(DownloadingPage&&) = delete;
+    DownloadingPage& operator=(DownloadingPage const&) = delete;
     ~DownloadingPage() override;
 
-    TR_DISABLE_COPY_MOVE(DownloadingPage)
-
 private:
     void on_core_prefs_changed(tr_quark key);
 
@@ -459,9 +470,11 @@ class SeedingPage : public PageBase
 {
 public:
     SeedingPage(BaseObjectType* cast_item, Glib::RefPtr<Gtk::Builder> const& builder, Glib::RefPtr<Session> const& core);
+    SeedingPage(SeedingPage&&) = delete;
+    SeedingPage(SeedingPage const&) = delete;
+    SeedingPage& operator=(SeedingPage&&) = delete;
+    SeedingPage& operator=(SeedingPage const&) = delete;
     ~SeedingPage() override = default;
-
-    TR_DISABLE_COPY_MOVE(SeedingPage)
 };
 
 SeedingPage::SeedingPage(
@@ -486,9 +499,11 @@ class DesktopPage : public PageBase
 {
 public:
     DesktopPage(BaseObjectType* cast_item, Glib::RefPtr<Gtk::Builder> const& builder, Glib::RefPtr<Session> const& core);
+    DesktopPage(DesktopPage&&) = delete;
+    DesktopPage(DesktopPage const&) = delete;
+    DesktopPage& operator=(DesktopPage&&) = delete;
+    DesktopPage& operator=(DesktopPage const&) = delete;
     ~DesktopPage() override = default;
-
-    TR_DISABLE_COPY_MOVE(DesktopPage)
 };
 
 DesktopPage::DesktopPage(
@@ -523,10 +538,12 @@ class PrivacyPage : public PageBase
 
 public:
     PrivacyPage(BaseObjectType* cast_item, Glib::RefPtr<Gtk::Builder> const& builder, Glib::RefPtr<Session> const& core);
+    PrivacyPage(PrivacyPage&&) = delete;
+    PrivacyPage(PrivacyPage const&) = delete;
+    PrivacyPage& operator=(PrivacyPage&&) = delete;
+    PrivacyPage& operator=(PrivacyPage const&) = delete;
     ~PrivacyPage() override;
 
-    TR_DISABLE_COPY_MOVE(PrivacyPage)
-
 private:
     void updateBlocklistText();
     void onBlocklistUpdated(bool success);
@@ -548,7 +565,7 @@ void PrivacyPage::updateBlocklistText()
 {
     int const n = tr_blocklistGetRuleCount(core_->get_session());
     auto const msg = fmt::format(
-        ngettext("Blocklist has {count:L} entry", "Blocklist has {count:L} entries", n),
+        fmt::runtime(ngettext("Blocklist has {count:L} entry", "Blocklist has {count:L} entries", n)),
         fmt::arg("count", n));
     label_->set_text(msg);
 }
@@ -635,10 +652,12 @@ class RemotePage : public PageBase
 
 public:
     RemotePage(BaseObjectType* cast_item, Glib::RefPtr<Gtk::Builder> const& builder, Glib::RefPtr<Session> const& core);
+    RemotePage(RemotePage&&) = delete;
+    RemotePage(RemotePage const&) = delete;
+    RemotePage& operator=(RemotePage&&) = delete;
+    RemotePage& operator=(RemotePage const&) = delete;
     ~RemotePage() override = default;
 
-    TR_DISABLE_COPY_MOVE(RemotePage)
-
 private:
     void refreshWhitelist();
     void onAddressEdited(Glib::ustring const& path, Glib::ustring const& address);
@@ -795,10 +814,10 @@ RemotePage::RemotePage(BaseObjectType* cast_item, Glib::RefPtr<Gtk::Builder> con
         store_ = whitelist_tree_model_new(gtr_pref_string_get(TR_KEY_rpc_whitelist));
 
         view_->set_model(store_);
-        setup_tree_view_button_event_handling(
+        setup_item_view_button_event_handling(
             *view_,
             {},
-            [this](double view_x, double view_y) { return on_tree_view_button_released(*view_, view_x, view_y); });
+            [this](double view_x, double view_y) { return on_item_view_button_released(*view_, view_x, view_y); });
 
         whitelist_widgets_.push_back(view_);
         auto const sel = view_->get_selection();
@@ -832,28 +851,32 @@ class SpeedPage : public PageBase
 {
 public:
     SpeedPage(BaseObjectType* cast_item, Glib::RefPtr<Gtk::Builder> const& builder, Glib::RefPtr<Session> const& core);
+    SpeedPage(SpeedPage&&) = delete;
+    SpeedPage(SpeedPage const&) = delete;
+    SpeedPage& operator=(SpeedPage&&) = delete;
+    SpeedPage& operator=(SpeedPage const&) = delete;
     ~SpeedPage() override = default;
-
-    TR_DISABLE_COPY_MOVE(SpeedPage)
 };
 
 SpeedPage::SpeedPage(BaseObjectType* cast_item, Glib::RefPtr<Gtk::Builder> const& builder, Glib::RefPtr<Session> const& core)
     : PageBase(cast_item, builder, core)
 {
+    auto const speed_units_kbyps_str = Speed::units().display_name(Speed::Units::KByps);
+
     localize_label(
         *init_check_button("upload_limit_check", TR_KEY_speed_limit_up_enabled),
-        fmt::arg("speed_units", speed_K_str));
+        fmt::arg("speed_units", speed_units_kbyps_str));
     init_spin_button("upload_limit_spin", TR_KEY_speed_limit_up, 0, std::numeric_limits<int>::max(), 5);
 
     localize_label(
         *init_check_button("download_limit_check", TR_KEY_speed_limit_down_enabled),
-        fmt::arg("speed_units", speed_K_str));
+        fmt::arg("speed_units", speed_units_kbyps_str));
     init_spin_button("download_limit_spin", TR_KEY_speed_limit_down, 0, std::numeric_limits<int>::max(), 5);
 
-    localize_label(*get_widget<Gtk::Label>("alt_upload_limit_label"), fmt::arg("speed_units", speed_K_str));
+    localize_label(*get_widget<Gtk::Label>("alt_upload_limit_label"), fmt::arg("speed_units", speed_units_kbyps_str));
     init_spin_button("alt_upload_limit_spin", TR_KEY_alt_speed_up, 0, std::numeric_limits<int>::max(), 5);
 
-    localize_label(*get_widget<Gtk::Label>("alt_download_limit_label"), fmt::arg("speed_units", speed_K_str));
+    localize_label(*get_widget<Gtk::Label>("alt_download_limit_label"), fmt::arg("speed_units", speed_units_kbyps_str));
     init_spin_button("alt_download_limit_spin", TR_KEY_alt_speed_down, 0, std::numeric_limits<int>::max(), 5);
 
     init_time_combo("alt_speed_start_time_combo", TR_KEY_alt_speed_time_begin);
@@ -870,15 +893,30 @@ class NetworkPage : public PageBase
 {
 public:
     NetworkPage(BaseObjectType* cast_item, Glib::RefPtr<Gtk::Builder> const& builder, Glib::RefPtr<Session> const& core);
+    NetworkPage(NetworkPage&&) = delete;
+    NetworkPage(NetworkPage const&) = delete;
+    NetworkPage& operator=(NetworkPage&&) = delete;
+    NetworkPage& operator=(NetworkPage const&) = delete;
     ~NetworkPage() override;
 
-    TR_DISABLE_COPY_MOVE(NetworkPage)
-
 private:
+    enum PortTestStatus : uint8_t
+    {
+        PORT_TEST_UNKNOWN = 0U,
+        PORT_TEST_CHECKING,
+        PORT_TEST_OPEN,
+        PORT_TEST_CLOSED,
+        PORT_TEST_ERROR
+    };
+
+    void portTestSetSensitive();
+    void updatePortStatusText();
     void onCorePrefsChanged(tr_quark key);
-    void onPortTested(bool isOpen);
+    void onPortTested(std::optional<bool> result, Session::PortTestIpProtocol ip_protocol);
     void onPortTest();
 
+    static std::string_view getPortStatusText(PortTestStatus status) noexcept;
+
 private:
     Glib::RefPtr<Session> core_;
 
@@ -888,15 +926,18 @@ private:
 
     sigc::connection portTag_;
     sigc::connection prefsTag_;
+
+    std::array<PortTestStatus, Session::NUM_PORT_TEST_IP_PROTOCOL> portTestStatus_ = {};
 };
 
 void NetworkPage::onCorePrefsChanged(tr_quark const key)
 {
     if (key == TR_KEY_peer_port)
     {
-        gtr_label_set_text(*portLabel_, _("Status unknown"));
-        portButton_->set_sensitive(true);
-        portSpin_->set_sensitive(true);
+        portTestStatus_[Session::PORT_TEST_IPV4] = PORT_TEST_UNKNOWN;
+        portTestStatus_[Session::PORT_TEST_IPV6] = PORT_TEST_UNKNOWN;
+        updatePortStatusText();
+        portTestSetSensitive();
     }
 }
 
@@ -906,28 +947,92 @@ NetworkPage::~NetworkPage()
     portTag_.disconnect();
 }
 
-void NetworkPage::onPortTested(bool isOpen)
+std::string_view NetworkPage::getPortStatusText(PortTestStatus const status) noexcept
 {
-    portLabel_->set_markup(fmt::format(
-        isOpen ? _("Port is {markup_begin}open{markup_end}") : _("Port is {markup_begin}closed{markup_end}"),
-        fmt::arg("markup_begin", "<b>"),
-        fmt::arg("markup_end", "</b>")));
-    portButton_->set_sensitive(true);
-    portSpin_->set_sensitive(true);
+    switch (status)
+    {
+    case PORT_TEST_UNKNOWN:
+        return C_("Port test status", "unknown");
+    case PORT_TEST_CHECKING:
+        return C_("Port test status", "checking…");
+    case PORT_TEST_OPEN:
+        return C_("Port test status", "open");
+    case PORT_TEST_CLOSED:
+        return C_("Port test status", "closed");
+    case PORT_TEST_ERROR:
+        return C_("Port test status", "error");
+    default:
+        return {};
+    }
+}
+
+void NetworkPage::updatePortStatusText()
+{
+    auto const status_ipv4 = getPortStatusText(portTestStatus_[Session::PORT_TEST_IPV4]);
+    auto const status_ipv6 = getPortStatusText(portTestStatus_[Session::PORT_TEST_IPV6]);
+
+    portLabel_->set_markup(
+        portTestStatus_[Session::PORT_TEST_IPV4] == portTestStatus_[Session::PORT_TEST_IPV6] ?
+            fmt::format(fmt::runtime(_("Status: <b>{status}</b>")), fmt::arg("status", status_ipv4)) :
+            fmt::format(
+                fmt::runtime(_("Status: <b>{status_ipv4}</b> (IPv4), <b>{status_ipv6}</b> (IPv6)")),
+                fmt::arg("status_ipv4", status_ipv4),
+                fmt::arg("status_ipv6", status_ipv6)));
+}
+
+void NetworkPage::portTestSetSensitive()
+{
+    // Depend on the RPC call status instead of the UI status, so that the widgets
+    // won't be enabled even if the port peer port changed while we have port_test
+    // RPC call(s) in-flight.
+    auto const sensitive = !core_->port_test_pending(Session::PORT_TEST_IPV4) &&
+        !core_->port_test_pending(Session::PORT_TEST_IPV6);
+    portButton_->set_sensitive(sensitive);
+    portSpin_->set_sensitive(sensitive);
+}
+
+void NetworkPage::onPortTested(std::optional<bool> const result, Session::PortTestIpProtocol const ip_protocol)
+{
+    auto constexpr ResultToStatus = [](std::optional<bool> const res)
+    {
+        if (!res)
+        {
+            return PORT_TEST_ERROR;
+        }
+        if (!*res)
+        {
+            return PORT_TEST_CLOSED;
+        }
+        return PORT_TEST_OPEN;
+    };
+
+    // Only update the UI if the current status is "checking", so that
+    // we won't show the port test results for the old peer port if it
+    // changed while we have port_test RPC call(s) in-flight.
+    if (auto& status = portTestStatus_[ip_protocol]; status == PORT_TEST_CHECKING)
+    {
+        status = ResultToStatus(result);
+        updatePortStatusText();
+    }
+    portTestSetSensitive();
 }
 
 void NetworkPage::onPortTest()
 {
-    portButton_->set_sensitive(false);
-    portSpin_->set_sensitive(false);
-    portLabel_->set_text(_("Testing TCP port…"));
+    portTestStatus_[Session::PORT_TEST_IPV4] = PORT_TEST_CHECKING;
+    portTestStatus_[Session::PORT_TEST_IPV6] = PORT_TEST_CHECKING;
+    updatePortStatusText();
 
     if (!portTag_.connected())
     {
-        portTag_ = core_->signal_port_tested().connect([this](bool is_open) { onPortTested(is_open); });
+        portTag_ = core_->signal_port_tested().connect(
+            [this](std::optional<bool> status, Session::PortTestIpProtocol ip_protocol) { onPortTested(status, ip_protocol); });
     }
 
-    core_->port_test();
+    core_->port_test(Session::PORT_TEST_IPV4);
+    core_->port_test(Session::PORT_TEST_IPV6);
+
+    portTestSetSensitive();
 }
 
 NetworkPage::NetworkPage(
@@ -941,6 +1046,7 @@ NetworkPage::NetworkPage(
     , portSpin_(init_spin_button("listening_port_spin", TR_KEY_peer_port, 1, std::numeric_limits<uint16_t>::max(), 1))
 {
     portButton_->signal_clicked().connect([this]() { onPortTest(); });
+    updatePortStatusText();
 
     prefsTag_ = core_->signal_prefs_changed().connect([this](auto key) { onCorePrefsChanged(key); });
 
