@@ -141,6 +141,11 @@ public:
         virtual ~Mediator() noexcept = default;
         virtual void sendto(void const* buf, size_t buflen, sockaddr const* addr, socklen_t addrlen) = 0;
         [[nodiscard]] virtual std::optional<tr_address> announce_ip() const = 0;
+        [[nodiscard]] virtual std::optional<tr_socket_address> dns_lookup(
+            tr_address_type ip_protocol,
+            std::string_view name,
+            uint16_t service,
+            std::string_view log_name) const;
     };
 
     virtual ~tr_announcer_udp() noexcept = default;
