@@ -1,4 +1,4 @@
-// This file Copyright © 2010-2023 Mnemosyne LLC.
+// This file Copyright © Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
@@ -6,31 +6,43 @@
 #include <array>
 #include <cerrno>
 #include <chrono>
-#include <cstdio>
-#include <map>
+#include <cstddef> // size_t
+#include <iterator> // back_insert_iterator, empty
 #include <mutex>
+#include <optional>
 #include <string>
 #include <string_view>
+#include <type_traits>
 #include <utility>
 
-#include <fmt/chrono.h>
-#include <fmt/format.h>
-
-#include "transmission.h"
-
-#include "file.h"
-#include "log.h"
-#include "tr-assert.h"
-#include "utils.h"
+#ifdef _WIN32
+#include <windows.h> // GetTimeZoneInformation
+#endif
 
 #ifdef __ANDROID__
 #include <android/log.h>
 #endif
 
+#include <fmt/chrono.h>
+#include <fmt/format.h>
+
+#include <small/map.hpp>
+
+#include "libtransmission/file.h"
+#include "libtransmission/log.h"
+#include "libtransmission/tr-assert.h"
+#include "libtransmission/tr-macros.h"
+#include "libtransmission/utils.h"
+
 using namespace std::literals;
 
 namespace
 {
+template<typename, typename = void>
+inline constexpr bool HasTmGmtoffV = false;
+
+template<typename T>
+inline constexpr bool HasTmGmtoffV<T, std::void_t<decltype(std::declval<T>().tm_gmtoff)>> = true;
 
 class tr_log_state
 {
@@ -48,7 +60,7 @@ public:
 
     tr_log_message** queue_tail_ = &queue_;
 
-    int queue_length_ = 0;
+    size_t queue_length_ = 0;
 
     std::recursive_mutex message_mutex_;
 };
@@ -57,39 +69,11 @@ auto log_state = tr_log_state{};
 
 // ---
 
-tr_sys_file_t tr_logGetFile()
-{
-    static bool initialized = false;
-    static tr_sys_file_t file = TR_BAD_SYS_FILE;
-
-    if (!initialized)
-    {
-        switch (tr_env_get_int("TR_DEBUG_FD", 0))
-        {
-        case 1:
-            file = tr_sys_file_get_std(TR_STD_SYS_FILE_OUT);
-            break;
-
-        case 2:
-            file = tr_sys_file_get_std(TR_STD_SYS_FILE_ERR);
-            break;
-
-        default:
-            file = TR_BAD_SYS_FILE;
-            break;
-        }
-
-        initialized = true;
-    }
-
-    return file;
-}
-
 void logAddImpl(
     [[maybe_unused]] std::string_view file,
     [[maybe_unused]] long line,
     [[maybe_unused]] tr_log_level level,
-    std::string_view msg,
+    std::string&& msg,
     [[maybe_unused]] std::string_view name)
 {
     if (std::empty(msg))
@@ -133,12 +117,12 @@ void logAddImpl(
 
 #else
 
-    if (tr_logGetQueueEnabled())
+    if (log_state.queue_enabled_)
     {
         auto* const newmsg = new tr_log_message{};
         newmsg->level = level;
-        newmsg->when = tr_time();
-        newmsg->message = msg;
+        newmsg->when = std::chrono::system_clock::now();
+        newmsg->message = std::move(msg);
         newmsg->file = file;
         newmsg->line = line;
         newmsg->name = name;
@@ -147,37 +131,29 @@ void logAddImpl(
         log_state.queue_tail_ = &newmsg->next;
         ++log_state.queue_length_;
 
-        if (log_state.queue_length_ > TR_LOG_MAX_QUEUE_LENGTH)
+        if (log_state.queue_length_ > TrLogMaxQueueLength)
         {
             tr_log_message* old = log_state.queue_;
             log_state.queue_ = old->next;
             old->next = nullptr;
             tr_logFreeQueue(old);
             --log_state.queue_length_;
-            TR_ASSERT(log_state.queue_length_ == TR_LOG_MAX_QUEUE_LENGTH);
+            TR_ASSERT(log_state.queue_length_ == TrLogMaxQueueLength);
         }
     }
     else
     {
-        tr_sys_file_t fp = tr_logGetFile();
+        auto buf = std::array<char, 64U>{};
+        auto const timestr = tr_logGetTimeStr(std::data(buf), std::size(buf));
 
-        if (fp == TR_BAD_SYS_FILE)
+        if (std::empty(name))
         {
-            fp = tr_sys_file_get_std(TR_STD_SYS_FILE_ERR);
-            // if fp is still bad, do not write log
-            if (fp == TR_BAD_SYS_FILE)
-            {
-                return;
-            }
+            fmt::print(stderr, "[{:s}] {:s}\n", timestr, msg);
+        }
+        else
+        {
+            fmt::print("[{:s}] {:s}: {:s}\n", timestr, name, msg);
         }
-
-        auto timestr = std::array<char, 64>{};
-        tr_logGetTimeStr(std::data(timestr), std::size(timestr));
-        tr_sys_file_write_line(
-            fp,
-            !std::empty(name) ? fmt::format(FMT_STRING("[{:s}] {:s}: {:s}"), std::data(timestr), name, msg) :
-                                fmt::format(FMT_STRING("[{:s}] {:s}"), std::data(timestr), msg));
-        tr_sys_file_flush(fp);
     }
 #endif
 }
@@ -204,11 +180,6 @@ void tr_logSetQueueEnabled(bool is_enabled)
     log_state.queue_enabled_ = is_enabled;
 }
 
-bool tr_logGetQueueEnabled()
-{
-    return log_state.queue_enabled_;
-}
-
 tr_log_message* tr_logGetQueue()
 {
     auto const lock = log_state.unique_lock();
@@ -233,20 +204,42 @@ void tr_logFreeQueue(tr_log_message* freeme)
 
 // ---
 
-char* tr_logGetTimeStr(char* buf, size_t buflen)
+std::string_view tr_logGetTimeStr(std::chrono::system_clock::time_point const now, char* const buf, size_t const buflen)
 {
-    auto const a = std::chrono::system_clock::now();
-    auto const [out, len] = fmt::format_to_n(
-        buf,
-        buflen - 1,
-        "{0:%F %H:%M:}{1:%S}",
-        a,
-        std::chrono::duration_cast<std::chrono::milliseconds>(a.time_since_epoch()));
-    *out = '\0';
-    return buf;
+    auto* walk = buf;
+    auto const now_time_t = std::chrono::system_clock::to_time_t(now);
+    auto const now_tm = *std::localtime(&now_time_t);
+    static bool constexpr HasTmGmtoff = HasTmGmtoffV<std::tm>;
+    static auto constexpr Fmt = HasTmGmtoff ? "{0:%FT%R:}{1:%S}{0:%z}"sv : "{0:%FT%R:}{1:%S}"sv;
+    walk = fmt::format_to_n(walk, buflen, Fmt, now_tm, std::chrono::time_point_cast<std::chrono::milliseconds>(now)).out;
+#ifdef _WIN32
+    if (auto tz_info = TIME_ZONE_INFORMATION{}; GetTimeZoneInformation(&tz_info) != TIME_ZONE_ID_INVALID)
+    {
+        // https://learn.microsoft.com/en-us/windows/win32/api/timezoneapi/nf-timezoneapi-gettimezoneinformation
+        // All translations between UTC time and local time are based on the following formula:
+        //     UTC = local time + bias
+        // The bias is the difference, in minutes, between UTC time and local time.
+        auto const offset = tz_info.Bias < 0 ? -tz_info.Bias : tz_info.Bias;
+        walk = fmt::format_to_n(
+                   walk,
+                   buflen - (walk - buf),
+                   "{:c}{:02d}{:02d}",
+                   tz_info.Bias < 0 ? '+' : '-',
+                   offset / 60,
+                   offset % 60)
+                   .out;
+    }
+#endif
+    return { buf, static_cast<size_t>(walk - buf) };
 }
 
-void tr_logAddMessage(char const* file, long line, tr_log_level level, std::string_view msg, std::string_view name)
+std::string_view tr_logGetTimeStr(char* buf, size_t buflen)
+{
+    auto const a = std::chrono::system_clock::now();
+    return tr_logGetTimeStr(a, buf, buflen);
+}
+
+void tr_logAddMessage(char const* file, long line, tr_log_level level, std::string&& msg, std::string_view name)
 {
     TR_ASSERT(!std::empty(msg));
 
@@ -260,7 +253,7 @@ void tr_logAddMessage(char const* file, long line, tr_log_level level, std::stri
     auto name_fallback = std::string{};
     if (std::empty(name))
     {
-        name_fallback = fmt::format(FMT_STRING("{}:{}"), filename, line);
+        name_fallback = fmt::format("{:s}:{:d}", filename, line);
         name = name_fallback;
     }
 
@@ -277,12 +270,12 @@ void tr_logAddMessage(char const* file, long line, tr_log_level level, std::stri
     auto const lock = log_state.unique_lock();
 
     // don't log the same warning ad infinitum.
-    // it's not useful after some point.
+    // at some point, it stops being useful.
     bool last_one = false;
     if (level == TR_LOG_CRITICAL || level == TR_LOG_ERROR || level == TR_LOG_WARN)
     {
         static auto constexpr MaxRepeat = size_t{ 30 };
-        static auto counts = new std::map<std::pair<std::string_view, int>, size_t>{};
+        static auto* const counts = new small::map<std::pair<std::string_view, long>, size_t>{};
 
         auto& count = (*counts)[std::make_pair(filename, line)];
         ++count;
@@ -295,15 +288,11 @@ void tr_logAddMessage(char const* file, long line, tr_log_level level, std::stri
     }
 
     // log the messages
-    logAddImpl(filename, line, level, msg, name);
+    logAddImpl(filename, line, level, std::move(msg), name);
     if (last_one)
     {
-        logAddImpl(
-            filename,
-            line,
-            level,
-            _("Too many messages like this! I won't log this message anymore this session."),
-            name);
+        char const* final_msg = _("Too many messages like this! I won't log this message anymore this session.");
+        logAddImpl(filename, line, level, final_msg, name);
     }
 
     errno = err;
@@ -341,7 +330,7 @@ static_assert(keysAreOrdered());
 
 std::optional<tr_log_level> tr_logGetLevelFromKey(std::string_view key_in)
 {
-    auto const key = tr_strlower(tr_strvStrip(key_in));
+    auto const key = tr_strlower(tr_strv_strip(key_in));
 
     for (auto const& [name, level] : LogKeys)
     {
