@@ -1,31 +1,40 @@
-// This file Copyright © 2007-2023 Mnemosyne LLC.
+// This file Copyright © Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
 
-#include <algorithm>
-#include <array>
 #include <cerrno>
 #include <cstdint>
-#include <cstring>
-#include <string>
+#include <functional>
+#include <type_traits> // std::underlying_type_t
+
+#ifdef _WIN32
+#include <ws2tcpip.h>
+#else
+#include <arpa/inet.h> // ntohl, ntohs
+#endif
 
 #include <event2/event.h>
-#include <event2/bufferevent.h>
 
 #include <libutp/utp.h>
 
 #include <fmt/format.h>
 
-#include "transmission.h"
-#include "session.h"
-#include "bandwidth.h"
-#include "log.h"
-#include "net.h"
-#include "peer-io.h"
-#include "tr-assert.h"
-#include "tr-utp.h"
-#include "utils.h" // for _()
+#include "libtransmission/transmission.h"
+
+#include "libtransmission/bandwidth.h"
+#include "libtransmission/block-info.h" // tr_block_info
+#include "libtransmission/error.h"
+#include "libtransmission/log.h"
+#include "libtransmission/net.h"
+#include "libtransmission/peer-io.h"
+#include "libtransmission/peer-socket.h" // tr_peer_socket, tr_netOpen...
+#include "libtransmission/session.h"
+#include "libtransmission/timer.h"
+#include "libtransmission/tr-assert.h"
+#include "libtransmission/utils.h" // for _()
+
+struct sockaddr;
 
 #define tr_logAddErrorIo(io, msg) tr_logAddError(msg, (io)->display_name())
 #define tr_logAddWarnIo(io, msg) tr_logAddWarn(msg, (io)->display_name())
@@ -36,7 +45,7 @@ namespace
 {
 // Helps us to ignore errors that say "try again later"
 // since that's what peer-io does by default anyway.
-[[nodiscard]] auto constexpr canRetryFromError(int error_code) noexcept
+[[nodiscard]] constexpr auto can_retry_from_error(int error_code) noexcept
 {
 #ifdef _WIN32
     return error_code == 0 || error_code == WSAEWOULDBLOCK || error_code == WSAEINTR || error_code == WSAEINPROGRESS;
@@ -51,14 +60,14 @@ size_t get_desired_output_buffer_size(tr_peerIo const* io, uint64_t now)
     // this is all kind of arbitrary, but what seems to work well is
     // being large enough to hold the next 20 seconds' worth of input,
     // or a few blocks, whichever is bigger. OK to tweak this as needed.
-    static auto constexpr PeriodSecs = 15U;
+    static auto constexpr PeriodSecs = uint64_t{ 15U };
 
     // the 3 is an arbitrary number of blocks;
     // the .5 is to leave room for protocol messages
-    static auto constexpr Floor = static_cast<size_t>(tr_block_info::BlockSize * 3.5);
+    static auto constexpr Floor = static_cast<uint64_t>(tr_block_info::BlockSize * 3.5);
 
-    auto const current_speed_bytes_per_second = io->get_piece_speed_bytes_per_second(now, TR_UP);
-    return std::max(Floor, current_speed_bytes_per_second * PeriodSecs);
+    auto const current_speed = io->get_piece_speed(now, TR_UP);
+    return std::max(Floor, current_speed.base_quantity() * PeriodSecs);
 }
 } // namespace
 
@@ -66,20 +75,24 @@ size_t get_desired_output_buffer_size(tr_peerIo const* io, uint64_t now)
 
 tr_peerIo::tr_peerIo(
     tr_session* session,
+    tr_peer_socket&& socket,
+    tr_bandwidth* parent_bandwidth,
     tr_sha1_digest_t const* info_hash,
     bool is_incoming,
-    bool is_seed,
-    tr_bandwidth* parent_bandwidth)
+    bool client_is_seed)
     : bandwidth_{ parent_bandwidth }
     , info_hash_{ info_hash != nullptr ? *info_hash : tr_sha1_digest_t{} }
     , session_{ session }
-    , is_seed_{ is_seed }
+    , flush_outbuf_trigger_{ session->timerMaker().create() }
+    , client_is_seed_{ client_is_seed }
     , is_incoming_{ is_incoming }
 {
+    set_socket(std::move(socket));
 }
 
 std::shared_ptr<tr_peerIo> tr_peerIo::create(
     tr_session* session,
+    tr_peer_socket&& socket,
     tr_bandwidth* parent,
     tr_sha1_digest_t const* info_hash,
     bool is_incoming,
@@ -88,8 +101,18 @@ std::shared_ptr<tr_peerIo> tr_peerIo::create(
     TR_ASSERT(session != nullptr);
     auto lock = session->unique_lock();
 
-    auto io = std::make_shared<tr_peerIo>(session, info_hash, is_incoming, is_seed, parent);
-    io->bandwidth().setPeer(io);
+    auto const io = std::shared_ptr<tr_peerIo>{
+        new tr_peerIo{ session, std::move(socket), parent, info_hash, is_incoming, is_seed }
+    };
+    io->bandwidth().set_peer(io);
+    io->flush_outbuf_trigger_->set_callback(
+        [weak = io->weak_from_this()]
+        {
+            if (auto const ptr = weak.lock())
+            {
+                ptr->try_write(SIZE_MAX);
+            }
+        });
     tr_logAddTraceIo(io, fmt::format("bandwidth is {}; its parent is {}", fmt::ptr(&io->bandwidth()), fmt::ptr(parent)));
     return io;
 }
@@ -97,54 +120,54 @@ std::shared_ptr<tr_peerIo> tr_peerIo::create(
 std::shared_ptr<tr_peerIo> tr_peerIo::new_incoming(tr_session* session, tr_bandwidth* parent, tr_peer_socket socket)
 {
     TR_ASSERT(session != nullptr);
-
-    auto peer_io = tr_peerIo::create(session, parent, nullptr, true, false);
-    peer_io->set_socket(std::move(socket));
-    return peer_io;
+    return tr_peerIo::create(session, std::move(socket), parent, nullptr, true, false);
 }
 
 std::shared_ptr<tr_peerIo> tr_peerIo::new_outgoing(
     tr_session* session,
     tr_bandwidth* parent,
-    tr_address const& addr,
-    tr_port port,
+    tr_socket_address const& socket_address,
     tr_sha1_digest_t const& info_hash,
-    bool is_seed,
+    bool client_is_seed,
     bool utp)
 {
-    TR_ASSERT(!tr_peer_socket::limit_reached(session));
     TR_ASSERT(session != nullptr);
-    TR_ASSERT(addr.is_valid());
-    TR_ASSERT(utp || session->allowsTCP());
-
-    if (!addr.is_valid_for_peers(port))
-    {
-        return {};
-    }
-
-    auto peer_io = tr_peerIo::create(session, parent, &info_hash, false, is_seed);
+    TR_ASSERT(socket_address.is_valid());
+    TR_ASSERT(!std::empty(session->preferred_transports()));
 
+    // N.B. This array needs to be kept in the same order as
+    // the tr_preferred_transport enum.
+    auto const get_socket = std::array<std::function<tr_peer_socket()>, TR_NUM_PREFERRED_TRANSPORT>{
+        [&]() -> tr_peer_socket
+        {
 #ifdef WITH_UTP
-    if (utp)
-    {
-        auto* const sock = utp_create_socket(session->utp_context);
-        utp_set_userdata(sock, peer_io.get());
-        peer_io->set_socket(tr_peer_socket{ addr, port, sock });
-
-        auto const [ss, sslen] = addr.to_sockaddr(port);
-        if (utp_connect(sock, reinterpret_cast<sockaddr const*>(&ss), sslen) == 0)
-        {
-            return peer_io;
-        }
-    }
+            if (utp)
+            {
+                auto* const sock = utp_create_socket(session->utp_context);
+                auto const [ss, sslen] = socket_address.to_sockaddr();
+                if (utp_connect(sock, reinterpret_cast<sockaddr const*>(&ss), sslen) == 0)
+                {
+                    return { socket_address, sock };
+                }
+            }
 #endif
-
-    if (!peer_io->socket_.is_valid())
-    {
-        if (auto sock = tr_netOpenPeerSocket(session, addr, port, is_seed); sock.is_valid())
+            return {};
+        },
+        [&]() -> tr_peer_socket
         {
-            peer_io->set_socket(std::move(sock));
-            return peer_io;
+            if (auto sock = tr_net_open_peer_socket(session, socket_address, client_is_seed); sock != TR_BAD_SOCKET)
+            {
+                return { session, socket_address, sock };
+            }
+            return {};
+        }
+    };
+
+    for (auto const& transport : session->preferred_transports())
+    {
+        if (auto sock = get_socket[transport](); sock.is_valid())
+        {
+            return tr_peerIo::create(session, std::move(sock), parent, &info_hash, false, client_is_seed);
         }
     }
 
@@ -171,8 +194,8 @@ void tr_peerIo::set_socket(tr_peer_socket socket_in)
 
     if (socket_.is_tcp())
     {
-        event_read_.reset(event_new(session_->eventBase(), socket_.handle.tcp, EV_READ, &tr_peerIo::event_read_cb, this));
-        event_write_.reset(event_new(session_->eventBase(), socket_.handle.tcp, EV_WRITE, &tr_peerIo::event_write_cb, this));
+        event_read_.reset(event_new(session_->event_base(), socket_.handle.tcp, EV_READ, &tr_peerIo::event_read_cb, this));
+        event_write_.reset(event_new(session_->event_base(), socket_.handle.tcp, EV_WRITE, &tr_peerIo::event_write_cb, this));
     }
 #ifdef WITH_UTP
     else if (socket_.is_utp())
@@ -191,6 +214,11 @@ void tr_peerIo::close()
     socket_.close();
     event_write_.reset();
     event_read_.reset();
+    inbuf_.clear();
+    outbuf_.clear();
+    outbuf_info_.clear();
+    encrypt_disable();
+    decrypt_disable();
 }
 
 void tr_peerIo::clear()
@@ -203,29 +231,23 @@ void tr_peerIo::clear()
 
 bool tr_peerIo::reconnect()
 {
-    TR_ASSERT(!this->is_incoming());
-    TR_ASSERT(this->session_->allowsTCP());
+    TR_ASSERT(!is_incoming());
+    if (!session_->allowsTCP())
+    {
+        return false;
+    }
 
-    short int const pending_events = this->pending_events_;
+    auto const pending_events = pending_events_;
     event_disable(EV_READ | EV_WRITE);
 
     close();
 
-    if (tr_peer_socket::limit_reached(session_))
+    auto const s = tr_net_open_peer_socket(session_, socket_address(), client_is_seed());
+    if (s == TR_BAD_SOCKET)
     {
         return false;
     }
-
-    auto const [addr, port] = socket_address();
-    socket_ = tr_netOpenPeerSocket(session_, addr, port, is_seed());
-
-    if (!socket_.is_tcp())
-    {
-        return false;
-    }
-
-    this->event_read_.reset(event_new(session_->eventBase(), socket_.handle.tcp, EV_READ, event_read_cb, this));
-    this->event_write_.reset(event_new(session_->eventBase(), socket_.handle.tcp, EV_WRITE, event_write_cb, this));
+    set_socket({ session_, socket_address(), s });
 
     event_enable(pending_events);
 
@@ -237,21 +259,21 @@ bool tr_peerIo::reconnect()
 void tr_peerIo::did_write_wrapper(size_t bytes_transferred)
 {
     auto const keep_alive = shared_from_this();
+    auto const now = tr_time_msec();
 
-    while (bytes_transferred != 0 && !std::empty(outbuf_info_))
+    if (bytes_transferred > 0U)
+    {
+        bandwidth().notify_bandwidth_consumed(TR_UP, bytes_transferred, false, now);
+    }
+
+    while (bytes_transferred > 0U && !std::empty(outbuf_info_))
     {
         auto& [n_bytes_left, is_piece_data] = outbuf_info_.front();
+        auto const payload = std::min(n_bytes_left, bytes_transferred);
 
-        size_t const payload = std::min(uint64_t{ n_bytes_left }, uint64_t{ bytes_transferred });
-        /* For µTP sockets, the overhead is computed in utp_on_overhead. */
-        size_t const overhead = socket_.guess_packet_overhead(payload);
-        uint64_t const now = tr_time_msec();
-
-        bandwidth().notifyBandwidthConsumed(TR_UP, payload, is_piece_data, now);
-
-        if (overhead > 0)
+        if (is_piece_data)
         {
-            bandwidth().notifyBandwidthConsumed(TR_UP, overhead, false, now);
+            bandwidth().notify_bandwidth_consumed(TR_UP, payload, true, now);
         }
 
         if (did_write_ != nullptr)
@@ -261,7 +283,7 @@ void tr_peerIo::did_write_wrapper(size_t bytes_transferred)
 
         bytes_transferred -= payload;
         n_bytes_left -= payload;
-        if (n_bytes_left == 0)
+        if (n_bytes_left == 0U)
         {
             outbuf_info_.pop_front();
         }
@@ -272,7 +294,7 @@ size_t tr_peerIo::try_write(size_t max)
 {
     static auto constexpr Dir = TR_UP;
 
-    if (max == 0)
+    if (max == 0U)
     {
         return {};
     }
@@ -280,28 +302,26 @@ size_t tr_peerIo::try_write(size_t max)
     auto& buf = outbuf_;
     max = std::min(max, std::size(buf));
     max = bandwidth().clamp(Dir, max);
-    if (max == 0)
+    if (max == 0U)
     {
         set_enabled(Dir, false);
         return {};
     }
 
-    tr_error* error = nullptr;
+    auto error = tr_error{};
     auto const n_written = socket_.try_write(buf, max, &error);
     // enable further writes if there's more data to write
-    set_enabled(Dir, !std::empty(buf) && (error == nullptr || canRetryFromError(error->code)));
+    set_enabled(Dir, !std::empty(buf) && (!error || can_retry_from_error(error.code())));
 
-    if (error != nullptr)
+    if (error)
     {
-        if (!canRetryFromError(error->code))
+        if (!can_retry_from_error(error.code()))
         {
             tr_logAddTraceIo(
                 this,
-                fmt::format("try_write err: wrote:{}, errno:{} ({})", n_written, error->code, error->message));
-            call_error_callback(*error);
+                fmt::format("try_write err: wrote:{}, errno:{} ({})", n_written, error.code(), error.message()));
+            call_error_callback(error);
         }
-
-        tr_error_clear(&error);
     }
     else if (n_written > 0U)
     {
@@ -328,7 +348,7 @@ void tr_peerIo::event_write_cb([[maybe_unused]] evutil_socket_t fd, short /*even
 
 // ---
 
-void tr_peerIo::can_read_wrapper()
+void tr_peerIo::can_read_wrapper(size_t bytes_transferred)
 {
     // try to consume the input buffer
 
@@ -341,54 +361,53 @@ void tr_peerIo::can_read_wrapper()
     auto const keep_alive = shared_from_this();
 
     auto const now = tr_time_msec();
-    auto done = bool{ false };
-    auto err = bool{ false };
+    auto done = false;
+    auto err = false;
+
+    if (socket_.is_tcp() && bytes_transferred > 0U)
+    {
+        bandwidth().notify_bandwidth_consumed(TR_DOWN, bytes_transferred, false, now);
+    }
 
     // In normal conditions, only continue processing if we still have bandwidth
     // quota for it.
     //
-    // The read buffer will grow indefinitely if libutp or the TCP stack keeps buffering
-    // data faster than the bandwidth limit allows. To safeguard against that, we keep
+    // The read buffer will grow indefinitely if libutp keeps buffering data faster
+    // than the bandwidth limit allows. To safeguard against that, we keep
     // processing if the read buffer is more than twice as large as the target size.
-    while (!done && !err && (read_buffer_size() > RcvBuf * 2U || bandwidth().clamp(TR_DOWN, read_buffer_size()) != 0U))
+    while (!done && !err &&
+           (socket_.is_tcp() || read_buffer_size() > RcvBuf * 2U || bandwidth().clamp(TR_DOWN, read_buffer_size()) != 0U))
     {
-        size_t piece = 0;
-        auto const old_len = read_buffer_size();
-        auto const read_state = can_read_ != nullptr ? can_read_(this, user_data_, &piece) : READ_ERR;
-        auto const used = old_len - read_buffer_size();
-        auto const overhead = socket_.guess_packet_overhead(used);
+        auto piece = size_t{};
+        auto const old_size = read_buffer_size();
+        auto const read_state = can_read_ != nullptr ? can_read_(this, user_data_, &piece) : ReadState::Err;
+        auto const used = old_size - read_buffer_size();
 
-        if (piece != 0)
+        if (piece > 0U)
         {
-            bandwidth().notifyBandwidthConsumed(TR_DOWN, piece, true, now);
+            bandwidth().notify_bandwidth_consumed(TR_DOWN, piece, true, now);
         }
 
-        if (used != piece)
+        if (socket_.is_utp() && used > 0U)
         {
-            bandwidth().notifyBandwidthConsumed(TR_DOWN, used - piece, false, now);
-        }
-
-        if (overhead > 0)
-        {
-            bandwidth().notifyBandwidthConsumed(TR_DOWN, overhead, false, now);
+            bandwidth().notify_bandwidth_consumed(TR_DOWN, used, false, now);
         }
 
         switch (read_state)
         {
-        case READ_NOW:
-            if (!std::empty(inbuf_))
+        case ReadState::Now:
+        case ReadState::Break:
+            if (std::empty(inbuf_))
             {
-                continue;
+                done = true;
             }
+            break;
 
+        case ReadState::Later:
             done = true;
             break;
 
-        case READ_LATER:
-            done = true;
-            break;
-
-        case READ_ERR:
+        case ReadState::Err:
             err = true;
             break;
         }
@@ -399,7 +418,7 @@ size_t tr_peerIo::try_read(size_t max)
 {
     static auto constexpr Dir = TR_DOWN;
 
-    if (max == 0)
+    if (max == 0U)
     {
         return {};
     }
@@ -407,30 +426,28 @@ size_t tr_peerIo::try_read(size_t max)
     // Do not write more than the bandwidth allows.
     // If there is no bandwidth left available, disable writes.
     max = bandwidth().clamp(Dir, max);
-    if (max == 0)
+    if (max == 0U)
     {
         set_enabled(Dir, false);
         return {};
     }
 
     auto& buf = inbuf_;
-    tr_error* error = nullptr;
-    auto const n_read = socket_.try_read(buf, max, &error);
-    set_enabled(Dir, error == nullptr || canRetryFromError(error->code));
+    auto error = tr_error{};
+    auto const n_read = socket_.try_read(buf, max, std::empty(buf), &error);
+    set_enabled(Dir, !error || can_retry_from_error(error.code()));
 
-    if (error != nullptr)
+    if (error)
     {
-        if (!canRetryFromError(error->code))
+        if (!can_retry_from_error(error.code()))
         {
-            tr_logAddTraceIo(this, fmt::format("try_read err: n_read:{} errno:{} ({})", n_read, error->code, error->message));
-            call_error_callback(*error);
+            tr_logAddTraceIo(this, fmt::format("try_read err: n_read:{} errno:{} ({})", n_read, error.code(), error.message()));
+            call_error_callback(error);
         }
-
-        tr_error_clear(&error);
     }
     else if (!std::empty(buf))
     {
-        can_read_wrapper();
+        can_read_wrapper(n_read);
     }
 
     return n_read;
@@ -450,7 +467,7 @@ void tr_peerIo::event_read_cb([[maybe_unused]] evutil_socket_t fd, short /*event
 
     // if we don't have any bandwidth left, stop reading
     auto const n_used = std::size(io->inbuf_);
-    auto const n_left = n_used >= MaxLen ? 0 : MaxLen - n_used;
+    auto const n_left = n_used >= MaxLen ? 0U : MaxLen - n_used;
     io->try_read(n_left);
 }
 
@@ -545,7 +562,7 @@ size_t tr_peerIo::flush(tr_direction dir, size_t limit)
 
 size_t tr_peerIo::flush_outgoing_protocol_msgs()
 {
-    size_t byte_count = 0;
+    size_t byte_count = 0U;
 
     /* count up how many bytes are used by non-piece-data messages
        at the front of our outbound queue */
@@ -562,6 +579,27 @@ size_t tr_peerIo::flush_outgoing_protocol_msgs()
     return flush(TR_UP, byte_count);
 }
 
+void tr_peerIo::flush_outbuf_soon()
+{
+    flush_outbuf_trigger_->start_single_shot(std::chrono::milliseconds::zero());
+}
+
+void tr_peerIo::write_bytes(void const* bytes, size_t n_bytes, bool is_piece_data)
+{
+    if (n_bytes == 0U)
+    {
+        return;
+    }
+
+    outbuf_info_.emplace_back(n_bytes, is_piece_data);
+
+    auto [resbuf, reslen] = outbuf_.reserve_space(n_bytes);
+    filter_.encrypt(reinterpret_cast<std::byte const*>(bytes), n_bytes, resbuf);
+    outbuf_.commit_space(n_bytes);
+
+    flush_outbuf_soon();
+}
+
 // ---
 
 size_t tr_peerIo::get_write_buffer_space(uint64_t now) const noexcept
@@ -571,41 +609,33 @@ size_t tr_peerIo::get_write_buffer_space(uint64_t now) const noexcept
     return desired_len > current_len ? desired_len - current_len : 0U;
 }
 
-void tr_peerIo::write(libtransmission::Buffer& buf, bool is_piece_data)
-{
-    auto [bytes, len] = buf.pullup();
-    encrypt(len, bytes);
-    outbuf_info_.emplace_back(std::size(buf), is_piece_data);
-    outbuf_.add(buf);
-}
-
-void tr_peerIo::write_bytes(void const* bytes, size_t n_bytes, bool is_piece_data)
-{
-    auto const old_size = std::size(outbuf_);
-
-    outbuf_.reserve(old_size + n_bytes);
-    outbuf_.add(bytes, n_bytes);
-
-    for (auto iter = std::begin(outbuf_) + old_size, end = std::end(outbuf_); iter != end; ++iter)
-    {
-        encrypt(1, &*iter);
-    }
-
-    outbuf_info_.emplace_back(n_bytes, is_piece_data);
-}
-
 // ---
 
-void tr_peerIo::read_bytes(void* bytes, size_t byte_count)
+void tr_peerIo::read_bytes(void* bytes, size_t n_bytes)
 {
-    TR_ASSERT(read_buffer_size() >= byte_count);
-
-    inbuf_.to_buf(bytes, byte_count);
-
-    if (is_encrypted())
+    auto walk = reinterpret_cast<std::byte*>(bytes);
+    n_bytes = std::min(n_bytes, std::size(inbuf_));
+    if (n_decrypt_remain_)
     {
-        decrypt(byte_count, bytes);
+        if (auto& n_remain = *n_decrypt_remain_; n_remain <= n_bytes)
+        {
+            filter_.decrypt(std::data(inbuf_), n_remain, walk);
+            inbuf_.drain(n_remain);
+            if (walk != nullptr)
+            {
+                walk += n_remain;
+            }
+            n_bytes -= n_remain;
+            filter_.decrypt_disable();
+            n_decrypt_remain_.reset();
+        }
+        else
+        {
+            n_remain -= n_bytes;
+        }
     }
+    filter_.decrypt(std::data(inbuf_), n_bytes, walk);
+    inbuf_.drain(n_bytes);
 }
 
 void tr_peerIo::read_uint16(uint16_t* setme)
@@ -622,18 +652,6 @@ void tr_peerIo::read_uint32(uint32_t* setme)
     *setme = ntohl(tmp);
 }
 
-void tr_peerIo::read_buffer_drain(size_t byte_count)
-{
-    auto buf = std::array<char, 4096>{};
-
-    while (byte_count > 0)
-    {
-        auto const this_pass = std::min(byte_count, std::size(buf));
-        read_bytes(std::data(buf), this_pass);
-        byte_count -= this_pass;
-    }
-}
-
 // --- UTP
 
 #ifdef WITH_UTP
@@ -643,7 +661,6 @@ void tr_peerIo::on_utp_state_change(int state)
     if (state == UTP_STATE_CONNECT)
     {
         tr_logAddTraceIo(this, "utp_on_state_change -- changed to connected");
-        utp_supported_ = true;
     }
     else if (state == UTP_STATE_WRITABLE)
     {
@@ -656,10 +673,9 @@ void tr_peerIo::on_utp_state_change(int state)
     }
     else if (state == UTP_STATE_EOF)
     {
-        tr_error* error = nullptr;
-        tr_error_set_from_errno(&error, ENOTCONN);
-        call_error_callback(*error);
-        tr_error_clear(&error);
+        auto error = tr_error{};
+        error.set_from_errno(ENOTCONN);
+        call_error_callback(error);
     }
     else if (state == UTP_STATE_DESTROYING)
     {
@@ -667,7 +683,7 @@ void tr_peerIo::on_utp_state_change(int state)
     }
     else
     {
-        tr_logAddErrorIo(this, fmt::format(_("Unknown state: {state}"), fmt::arg("state", state)));
+        tr_logAddErrorIo(this, fmt::format(fmt::runtime(_("Unknown state: {state}")), fmt::arg("state", state)));
     }
 }
 
@@ -680,23 +696,24 @@ void tr_peerIo::on_utp_error(int errcode)
         return;
     }
 
-    tr_error* error = nullptr;
+    auto error = tr_error{};
     switch (errcode)
     {
     case UTP_ECONNREFUSED:
-        tr_error_set_from_errno(&error, ECONNREFUSED);
+        error.set_from_errno(ECONNREFUSED);
         break;
     case UTP_ECONNRESET:
-        tr_error_set_from_errno(&error, ECONNRESET);
+        error.set_from_errno(ECONNRESET);
         break;
     case UTP_ETIMEDOUT:
-        tr_error_set_from_errno(&error, ETIMEDOUT);
+        error.set_from_errno(ETIMEDOUT);
         break;
     default:
-        tr_error_set(&error, errcode, utp_error_code_names[errcode]);
+        error.set(errcode, utp_error_code_names[errcode]);
+        break;
     }
-    call_error_callback(*error);
-    tr_error_clear(&error);
+
+    call_error_callback(error);
 }
 
 #endif /* #ifdef WITH_UTP */
@@ -723,7 +740,7 @@ void tr_peerIo::utp_init([[maybe_unused]] struct_utp_context* ctx)
 
                 io->inbuf_.add(args->buf, args->len);
                 io->set_enabled(TR_DOWN, true);
-                io->can_read_wrapper();
+                io->can_read_wrapper(args->len);
 
                 // utp_read_drained() notifies libutp that we read a packet from them.
                 // It opens up the congestion window by sending an ACK (soonish) if
@@ -757,19 +774,6 @@ void tr_peerIo::utp_init([[maybe_unused]] struct_utp_context* ctx)
             return {};
         });
 
-    utp_set_callback(
-        ctx,
-        UTP_ON_OVERHEAD_STATISTICS,
-        [](utp_callback_arguments* args) -> uint64
-        {
-            if (auto* const io = static_cast<tr_peerIo*>(utp_get_userdata(args->socket)); io != nullptr)
-            {
-                tr_logAddTraceIo(io, fmt::format("{:d} overhead bytes via utp", args->len));
-                io->bandwidth().notifyBandwidthConsumed(args->send != 0 ? TR_UP : TR_DOWN, args->len, false, tr_time_msec());
-            }
-            return {};
-        });
-
     utp_set_callback(
         ctx,
         UTP_ON_STATE_CHANGE,
