@@ -1,33 +1,37 @@
-// This file Copyright © 2017-2023 Mnemosyne LLC.
+// This file Copyright © Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
 
+#include <algorithm> // std::min
+#include <cerrno>
+#include <cstddef> // std::byte
+
 #include <fmt/format.h>
 
 #include <libutp/utp.h>
 
-#include "transmission.h"
-
-#include "peer-socket.h"
-#include "net.h"
-#include "session.h"
+#include "libtransmission/error.h"
+#include "libtransmission/log.h"
+#include "libtransmission/net.h"
+#include "libtransmission/peer-socket.h"
+#include "libtransmission/session.h"
+#include "libtransmission/tr-assert.h"
 
 #define tr_logAddErrorIo(io, msg) tr_logAddError(msg, (io)->display_name())
 #define tr_logAddWarnIo(io, msg) tr_logAddWarn(msg, (io)->display_name())
 #define tr_logAddDebugIo(io, msg) tr_logAddDebug(msg, (io)->display_name())
 #define tr_logAddTraceIo(io, msg) tr_logAddTrace(msg, (io)->display_name())
 
-tr_peer_socket::tr_peer_socket(tr_session const* session, tr_address const& address, tr_port port, tr_socket_t sock)
+tr_peer_socket::tr_peer_socket(tr_session const* session, tr_socket_address const& socket_address, tr_socket_t sock)
     : handle{ sock }
-    , address_{ address }
-    , port_{ port }
+    , socket_address_{ socket_address }
     , type_{ Type::TCP }
 {
     TR_ASSERT(sock != TR_BAD_SOCKET);
 
-    ++n_open_sockets_;
-    session->setSocketTOS(sock, address_.type);
+    ++n_open_sockets;
+    session->setSocketDiffServ(sock, address().type);
 
     if (auto const& algo = session->peerCongestionAlgorithm(); !std::empty(algo))
     {
@@ -37,14 +41,13 @@ tr_peer_socket::tr_peer_socket(tr_session const* session, tr_address const& addr
     tr_logAddTraceIo(this, fmt::format("socket (tcp) is {}", handle.tcp));
 }
 
-tr_peer_socket::tr_peer_socket(tr_address const& address, tr_port port, struct UTPSocket* const sock)
-    : address_{ address }
-    , port_{ port }
+tr_peer_socket::tr_peer_socket(tr_socket_address const& socket_address, struct UTPSocket* const sock)
+    : socket_address_{ socket_address }
     , type_{ Type::UTP }
 {
     TR_ASSERT(sock != nullptr);
 
-    ++n_open_sockets_;
+    ++n_open_sockets;
     handle.utp = sock;
 
     tr_logAddTraceIo(this, fmt::format("socket (µTP) is {}", fmt::ptr(handle.utp)));
@@ -54,13 +57,13 @@ void tr_peer_socket::close()
 {
     if (is_tcp() && (handle.tcp != TR_BAD_SOCKET))
     {
-        --n_open_sockets_;
+        --n_open_sockets;
         tr_net_close_socket(handle.tcp);
     }
 #ifdef WITH_UTP
     else if (is_utp())
     {
-        --n_open_sockets_;
+        --n_open_sockets;
         utp_set_userdata(handle.utp, nullptr);
         utp_close(handle.utp);
     }
@@ -70,7 +73,7 @@ void tr_peer_socket::close()
     handle = {};
 }
 
-size_t tr_peer_socket::try_write(Buffer& buf, size_t max, tr_error** error) const
+size_t tr_peer_socket::try_write(OutBuf& buf, size_t max, tr_error* error) const
 {
     if (max == size_t{})
     {
@@ -85,10 +88,10 @@ size_t tr_peer_socket::try_write(Buffer& buf, size_t max, tr_error** error) cons
 #ifdef WITH_UTP
     if (is_utp())
     {
-        auto const [data, datalen] = buf.pullup();
-
         errno = 0;
-        auto const n_written = utp_write(handle.utp, data, std::min(datalen, max));
+        // NB: utp_write() does not modify its 2nd arg, but a wart in
+        // libutp's public API requires it to be non-const anyway :shrug:
+        auto const n_written = utp_write(handle.utp, const_cast<std::byte*>(std::data(buf)), std::min(std::size(buf), max));
         auto const error_code = errno;
 
         if (n_written > 0)
@@ -97,9 +100,9 @@ size_t tr_peer_socket::try_write(Buffer& buf, size_t max, tr_error** error) cons
             return static_cast<size_t>(n_written);
         }
 
-        if (n_written < 0 && error_code != 0)
+        if (error != nullptr && n_written < 0 && error_code != 0)
         {
-            tr_error_set_from_errno(error, error_code);
+            error->set_from_errno(error_code);
         }
     }
 #endif
@@ -107,7 +110,7 @@ size_t tr_peer_socket::try_write(Buffer& buf, size_t max, tr_error** error) cons
     return {};
 }
 
-size_t tr_peer_socket::try_read(Buffer& buf, size_t max, tr_error** error) const
+size_t tr_peer_socket::try_read(InBuf& buf, size_t max, [[maybe_unused]] bool buf_is_empty, tr_error* error) const
 {
     if (max == size_t{})
     {
@@ -119,20 +122,14 @@ size_t tr_peer_socket::try_read(Buffer& buf, size_t max, tr_error** error) const
         return buf.add_socket(handle.tcp, max, error);
     }
 
-#ifdef WITH_UTP
-    // utp_read_drained() notifies libutp that this read buffer is empty.
-    // It opens up the congestion window by sending an ACK (soonish) if
-    // one was not going to be sent.
-    if (is_utp() && std::empty(buf))
-    {
-        utp_read_drained(handle.utp);
-    }
-#endif
+    // Unlike conventional BSD-style socket API, libutp pushes incoming data to the
+    // caller via a callback, instead of allowing the caller to pull data from
+    // its buffers. Therefore, reading data from a uTP socket is not handled here.
 
     return {};
 }
 
-bool tr_peer_socket::limit_reached(tr_session* const session) noexcept
+bool tr_peer_socket::limit_reached(tr_session const* const session) noexcept
 {
-    return n_open_sockets_.load() >= session->peerLimit();
+    return n_open_sockets.load() >= session->peerLimit();
 }
