@@ -19,8 +19,6 @@
 #include <fmt/chrono.h>
 #include <fmt/format.h>
 
-#include <small/map.hpp>
-
 #include "libtransmission/transmission.h"
 #include "libtransmission/tr-macros.h"
 
@@ -53,6 +51,25 @@ struct tr_ctor;
 using namespace std::literals;
 using namespace libtransmission::Values;
 
+#define tr_return_if_fail(expr) \
+    do \
+    { \
+        if (!(expr)) \
+        { \
+            tr_logAddWarn(#expr); \
+            return; \
+        } \
+    } while (0)
+#define tr_return_val_if_fail(expr, val) \
+    do \
+    { \
+        if (!(expr)) \
+        { \
+            tr_logAddWarn(#expr); \
+            return val; \
+        } \
+    } while (0)
+
 // ---
 
 void tr_torrent::Error::set_tracker_warning(tr_interned_string announce_url, std::string_view errmsg)
@@ -213,35 +230,37 @@ bool tr_torrentIsSeedRatioDone(tr_torrent const* tor)
 
 void tr_torrentSetSpeedLimit_KBps(tr_torrent* const tor, tr_direction const dir, size_t const limit_kbyps)
 {
+    tr_return_if_fail(tr_isTorrent(tor));
+
     tor->set_speed_limit(dir, Speed{ limit_kbyps, Speed::Units::KByps });
 }
 
 size_t tr_torrentGetSpeedLimit_KBps(tr_torrent const* const tor, tr_direction const dir)
 {
-    TR_ASSERT(tr_isTorrent(tor));
-    TR_ASSERT(tr_isDirection(dir));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
+    tr_return_val_if_fail(tr_isDirection(dir), {});
 
     return tor->speed_limit(dir).count(Speed::Units::KByps);
 }
 
 void tr_torrentUseSpeedLimit(tr_torrent* const tor, tr_direction const dir, bool const enabled)
 {
-    TR_ASSERT(tr_isTorrent(tor));
-    TR_ASSERT(tr_isDirection(dir));
+    tr_return_if_fail(tr_isTorrent(tor));
+    tr_return_if_fail(tr_isDirection(dir));
 
     tor->use_speed_limit(dir, enabled);
 }
 
 bool tr_torrentUsesSpeedLimit(tr_torrent const* const tor, tr_direction const dir)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
 
     return tor->uses_speed_limit(dir);
 }
 
 void tr_torrentUseSessionLimits(tr_torrent* const tor, bool const enabled)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_if_fail(tr_isTorrent(tor));
 
     if (tor->bandwidth().honor_parent_limits(TR_UP, enabled) || tor->bandwidth().honor_parent_limits(TR_DOWN, enabled))
     {
@@ -251,7 +270,7 @@ void tr_torrentUseSessionLimits(tr_torrent* const tor, bool const enabled)
 
 bool tr_torrentUsesSessionLimits(tr_torrent const* tor)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
 
     return tor->uses_session_limits();
 }
@@ -260,35 +279,35 @@ bool tr_torrentUsesSessionLimits(tr_torrent const* tor)
 
 void tr_torrentSetRatioMode(tr_torrent* const tor, tr_ratiolimit mode)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_if_fail(tr_isTorrent(tor));
 
     tor->set_seed_ratio_mode(mode);
 }
 
 tr_ratiolimit tr_torrentGetRatioMode(tr_torrent const* const tor)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
 
     return tor->seed_ratio_mode();
 }
 
 void tr_torrentSetRatioLimit(tr_torrent* const tor, double desired_ratio)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_if_fail(tr_isTorrent(tor));
 
     tor->set_seed_ratio(desired_ratio);
 }
 
 double tr_torrentGetRatioLimit(tr_torrent const* const tor)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
 
     return tor->seed_ratio();
 }
 
 bool tr_torrentGetSeedRatio(tr_torrent const* const tor, double* ratio)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
 
     auto const val = tor->effective_seed_ratio();
 
@@ -304,28 +323,28 @@ bool tr_torrentGetSeedRatio(tr_torrent const* const tor, double* ratio)
 
 void tr_torrentSetIdleMode(tr_torrent* const tor, tr_idlelimit mode)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_if_fail(tr_isTorrent(tor));
 
     tor->set_idle_limit_mode(mode);
 }
 
 tr_idlelimit tr_torrentGetIdleMode(tr_torrent const* const tor)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
 
     return tor->idle_limit_mode();
 }
 
 void tr_torrentSetIdleLimit(tr_torrent* const tor, uint16_t idle_minutes)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_if_fail(tr_isTorrent(tor));
 
     tor->set_idle_limit_minutes(idle_minutes);
 }
 
 uint16_t tr_torrentGetIdleLimit(tr_torrent const* const tor)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
 
     return tor->idle_limit_minutes();
 }
@@ -473,6 +492,8 @@ size_t tr_torrentGetQueuePosition(tr_torrent const* tor)
 
 void tr_torrentSetQueuePosition(tr_torrent* tor, size_t queue_position)
 {
+    tr_return_if_fail(tr_isTorrent(tor));
+
     tor->set_queue_position(queue_position);
 }
 
@@ -686,17 +707,10 @@ void tr_torrent::stop_now()
     set_is_queued(false);
 }
 
-void tr_torrentRemoveInSessionThread(
-    tr_torrent* tor,
-    bool delete_flag,
-    tr_fileFunc delete_func,
-    void* delete_user_data,
-    tr_torrent_remove_done_func callback,
-    void* callback_user_data)
+void tr_torrentRemoveInSessionThread(tr_torrent* tor, bool delete_flag, tr_fileFunc delete_func, void* delete_user_data)
 {
     auto const lock = tor->unique_lock();
 
-    bool ok = true;
     if (delete_flag && tor->has_metainfo())
     {
         // ensure the files are all closed and idle before moving
@@ -713,31 +727,20 @@ void tr_torrentRemoveInSessionThread(
             delete_func(filename, delete_user_data, nullptr);
         };
 
-        tr_error error;
+        auto error = tr_error{};
         tor->files().remove(tor->current_dir(), tor->name(), delete_func_wrapper, &error);
         if (error)
         {
-            ok = false;
-            tor->is_deleting_ = false;
-
-            tor->error().set_local_error(
+            tr_logAddWarnTor(
+                tor,
                 fmt::format(
                     fmt::runtime(_("Couldn't remove all torrent files: {error} ({error_code})")),
                     fmt::arg("error", error.message()),
                     fmt::arg("error_code", error.code())));
-            tr_torrentStop(tor);
         }
     }
 
-    if (callback != nullptr)
-    {
-        callback(tor->id(), ok, callback_user_data);
-    }
-
-    if (ok)
-    {
-        tr_torrentFreeInSessionThread(tor);
-    }
+    tr_torrentFreeInSessionThread(tor);
 }
 
 void tr_torrentStop(tr_torrent* tor)
@@ -754,28 +757,15 @@ void tr_torrentStop(tr_torrent* tor)
     tor->session->run_in_session_thread([tor]() { tor->stop_now(); });
 }
 
-void tr_torrentRemove(
-    tr_torrent* tor,
-    bool delete_flag,
-    tr_fileFunc delete_func,
-    void* delete_user_data,
-    tr_torrent_remove_done_func callback,
-    void* callback_user_data)
+void tr_torrentRemove(tr_torrent* tor, bool delete_flag, tr_fileFunc delete_func, void* delete_user_data)
 {
     using namespace start_stop_helpers;
 
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_if_fail(tr_isTorrent(tor));
 
     tor->is_deleting_ = true;
 
-    tor->session->run_in_session_thread(
-        tr_torrentRemoveInSessionThread,
-        tor,
-        delete_flag,
-        delete_func,
-        delete_user_data,
-        callback,
-        callback_user_data);
+    tor->session->run_in_session_thread(tr_torrentRemoveInSessionThread, tor, delete_flag, delete_func, delete_user_data);
 }
 
 void tr_torrentFreeInSessionThread(tr_torrent* tor)
@@ -783,7 +773,6 @@ void tr_torrentFreeInSessionThread(tr_torrent* tor)
     using namespace start_stop_helpers;
 
     TR_ASSERT(tr_isTorrent(tor));
-    TR_ASSERT(tor->session != nullptr);
     TR_ASSERT(tor->session->am_in_session_thread());
 
     if (!tor->session->isClosing())
@@ -1170,9 +1159,9 @@ void tr_torrent::set_location(std::string_view location, bool move_from_old_path
 
 void tr_torrentSetLocation(tr_torrent* tor, char const* location, bool move_from_old_path, int volatile* setme_state)
 {
-    TR_ASSERT(tr_isTorrent(tor));
-    TR_ASSERT(location != nullptr);
-    TR_ASSERT(*location != '\0');
+    tr_return_if_fail(tr_isTorrent(tor));
+    tr_return_if_fail(location != nullptr);
+    tr_return_if_fail(*location != '\0');
 
     tor->set_location(location, move_from_old_path, setme_state);
 }
@@ -1197,7 +1186,7 @@ bool tr_torrent::has_any_local_data() const
 
 void tr_torrentSetDownloadDir(tr_torrent* tor, char const* path)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_if_fail(tr_isTorrent(tor));
 
     if (tor->download_dir_ != path)
     {
@@ -1207,21 +1196,21 @@ void tr_torrentSetDownloadDir(tr_torrent* tor, char const* path)
 
 char const* tr_torrentGetDownloadDir(tr_torrent const* tor)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), "");
 
     return tor->download_dir().c_str();
 }
 
 char const* tr_torrentGetCurrentDir(tr_torrent const* tor)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), "");
 
     return tor->current_dir().c_str();
 }
 
 void tr_torrentChangeMyPort(tr_torrent* tor)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_if_fail(tr_isTorrent(tor));
 
     if (tor->is_running())
     {
@@ -1251,7 +1240,7 @@ void tr_torrentManualUpdate(tr_torrent* tor)
 {
     using namespace manual_update_helpers;
 
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_if_fail(tr_isTorrent(tor));
 
     tor->session->run_in_session_thread(torrentManualUpdateImpl, tor);
 }
@@ -1394,6 +1383,8 @@ tr_stat tr_torrent::stats() const
 
 tr_stat const* tr_torrentStat(tr_torrent* const tor)
 {
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
+
     tor->stats_ = tor->stats();
     return &tor->stats_;
 }
@@ -1402,7 +1393,7 @@ tr_stat const* tr_torrentStat(tr_torrent* const tor)
 
 tr_file_view tr_torrentFile(tr_torrent const* tor, tr_file_index_t file)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
 
     auto const& subpath = tor->file_subpath(file);
     auto const priority = tor->file_priorities_.file_priority(file);
@@ -1419,38 +1410,44 @@ tr_file_view tr_torrentFile(tr_torrent const* tor, tr_file_index_t file)
     return { subpath.c_str(), have, length, have >= length ? 1.0 : have / double(length), begin, end, priority, wanted };
 }
 
-size_t tr_torrentFileCount(tr_torrent const* torrent)
+size_t tr_torrentFileCount(tr_torrent const* tor)
 {
-    TR_ASSERT(tr_isTorrent(torrent));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
 
-    return torrent->file_count();
+    return tor->file_count();
 }
 
 tr_webseed_view tr_torrentWebseed(tr_torrent const* tor, size_t nth)
 {
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
+
     return tr_peerMgrWebseed(tor, nth);
 }
 
 size_t tr_torrentWebseedCount(tr_torrent const* tor)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
 
     return tor->webseed_count();
 }
 
 tr_tracker_view tr_torrentTracker(tr_torrent const* tor, size_t i)
 {
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
+
     return tr_announcerTracker(tor, i);
 }
 
 size_t tr_torrentTrackerCount(tr_torrent const* tor)
 {
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
+
     return tr_announcerTrackerCount(tor);
 }
 
 tr_torrent_view tr_torrentView(tr_torrent const* tor)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
 
     auto ret = tr_torrent_view{};
     ret.name = tor->name().c_str();
@@ -1470,11 +1467,15 @@ tr_torrent_view tr_torrentView(tr_torrent const* tor)
 
 std::string tr_torrentFilename(tr_torrent const* tor)
 {
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
+
     return std::string{ tor->torrent_file() };
 }
 
 size_t tr_torrentFilenameToBuf(tr_torrent const* tor, char* buf, size_t buflen)
 {
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
+
     return tr_strv_to_buf(tr_torrentFilename(tor), buf, buflen);
 }
 
@@ -1482,7 +1483,7 @@ size_t tr_torrentFilenameToBuf(tr_torrent const* tor, char* buf, size_t buflen)
 
 tr_peer_stat* tr_torrentPeers(tr_torrent const* tor, size_t* peer_count)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
 
     return tr_peerMgrPeerStats(tor, peer_count);
 }
@@ -1494,7 +1495,7 @@ void tr_torrentPeersFree(tr_peer_stat* peer_stats, size_t /*peer_count*/)
 
 void tr_torrentAvailability(tr_torrent const* tor, int8_t* tab, int size)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_if_fail(tr_isTorrent(tor));
 
     if (tab != nullptr && size > 0)
     {
@@ -1504,6 +1505,8 @@ void tr_torrentAvailability(tr_torrent const* tor, int8_t* tab, int size)
 
 void tr_torrentAmountFinished(tr_torrent const* tor, float* tabs, int n_tabs)
 {
+    tr_return_if_fail(tr_isTorrent(tor));
+
     tor->amount_done_bins(tabs, n_tabs);
 }
 
@@ -1511,26 +1514,26 @@ void tr_torrentAmountFinished(tr_torrent const* tor, float* tabs, int n_tabs)
 
 void tr_torrentStart(tr_torrent* tor)
 {
-    if (tr_isTorrent(tor))
-    {
-        tor->start_when_stable_ = true;
-        tor->start(false /*bypass_queue*/, {});
-    }
+    tr_return_if_fail(tr_isTorrent(tor));
+
+    tor->start_when_stable_ = true;
+    tor->start(false /*bypass_queue*/, {});
 }
 
 void tr_torrentStartNow(tr_torrent* tor)
 {
-    if (tr_isTorrent(tor))
-    {
-        tor->start_when_stable_ = true;
-        tor->start(true /*bypass_queue*/, {});
-    }
+    tr_return_if_fail(tr_isTorrent(tor));
+
+    tor->start_when_stable_ = true;
+    tor->start(true /*bypass_queue*/, {});
 }
 
 // ---
 
 void tr_torrentVerify(tr_torrent* tor)
 {
+    tr_return_if_fail(tr_isTorrent(tor));
+
     tor->session->run_in_session_thread(
         [tor, session = tor->session, tor_id = tor->id()]()
         {
@@ -1845,7 +1848,7 @@ void tr_torrent::recheck_completeness()
 
 void tr_torrentSetFileDLs(tr_torrent* tor, tr_file_index_t const* files, tr_file_index_t n_files, bool wanted)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_if_fail(tr_isTorrent(tor));
 
     tor->set_files_wanted(files, n_files, wanted);
 }
@@ -1895,15 +1898,15 @@ void tr_torrent::set_bandwidth_group(std::string_view group_name) noexcept
 
 tr_priority_t tr_torrentGetPriority(tr_torrent const* tor)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
 
     return tor->get_priority();
 }
 
 void tr_torrentSetPriority(tr_torrent* const tor, tr_priority_t const priority)
 {
-    TR_ASSERT(tr_isTorrent(tor));
-    TR_ASSERT(tr_isPriority(priority));
+    tr_return_if_fail(tr_isTorrent(tor));
+    tr_return_if_fail(tr_isPriority(priority));
 
     if (tor->bandwidth().get_priority() != priority)
     {
@@ -1917,14 +1920,14 @@ void tr_torrentSetPriority(tr_torrent* const tor, tr_priority_t const priority)
 
 void tr_torrentSetPeerLimit(tr_torrent* tor, uint16_t max_connected_peers)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_if_fail(tr_isTorrent(tor));
 
     tor->set_peer_limit(max_connected_peers);
 }
 
 uint16_t tr_torrentGetPeerLimit(tr_torrent const* tor)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
 
     return tor->peer_limit();
 }
@@ -2081,16 +2084,22 @@ void tr_torrent::on_tracker_response(tr_tracker_event const* event)
 
 bool tr_torrentSetTrackerList(tr_torrent* tor, char const* text)
 {
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
+
     return text != nullptr && tor->set_announce_list(text);
 }
 
 std::string tr_torrentGetTrackerList(tr_torrent const* tor)
 {
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
+
     return tor->announce_list().to_string();
 }
 
 size_t tr_torrentGetTrackerListToBuf(tr_torrent const* tor, char* buf, size_t buflen)
 {
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
+
     return tr_strv_to_buf(tr_torrentGetTrackerList(tor), buf, buflen);
 }
 
@@ -2098,7 +2107,7 @@ size_t tr_torrentGetTrackerListToBuf(tr_torrent const* tor, char* buf, size_t bu
 
 uint64_t tr_torrentGetBytesLeftToAllocate(tr_torrent const* tor)
 {
-    TR_ASSERT(tr_isTorrent(tor));
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
 
     uint64_t bytes_left = 0;
 
@@ -2126,30 +2135,7 @@ uint64_t tr_torrentGetBytesLeftToAllocate(tr_torrent const* tor)
 
 std::string_view tr_torrent::primary_mime_type() const
 {
-    // count up how many bytes there are for each mime-type in the torrent
-    // NB: get_mime_type_for_filename() always returns the same ptr for a
-    // mime_type, so its raw pointer can be used as a key.
-    auto size_per_mime_type = small::unordered_map<std::string_view, size_t, 256U>{};
-    for (tr_file_index_t i = 0, n = this->file_count(); i < n; ++i)
-    {
-        auto const mime_type = tr_get_mime_type_for_filename(this->file_subpath(i));
-        size_per_mime_type[mime_type] += this->file_size(i);
-    }
-
-    if (std::empty(size_per_mime_type))
-    {
-        // https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types/Common_types
-        // application/octet-stream is the default value for all other cases.
-        // An unknown file type should use this type.
-        auto constexpr Fallback = "application/octet-stream"sv;
-        return Fallback;
-    }
-
-    auto const it = std::max_element(
-        std::begin(size_per_mime_type),
-        std::end(size_per_mime_type),
-        [](auto const& a, auto const& b) { return a.second < b.second; });
-    return it->first;
+    return files().primary_mime_type();
 }
 
 // ---
@@ -2244,6 +2230,8 @@ void tr_torrent::on_block_received(tr_block_index_t const block)
 
 std::string tr_torrentFindFile(tr_torrent const* tor, tr_file_index_t file_num)
 {
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
+
     auto const found = tor->find_file(file_num);
     return std::string{ found ? found->filename().sv() : ""sv };
 }
@@ -2512,6 +2500,8 @@ void tr_torrentRenamePath(
     tr_torrent_rename_done_func callback,
     void* callback_user_data)
 {
+    tr_return_if_fail(tr_isTorrent(tor));
+
     oldpath = oldpath != nullptr ? oldpath : "";
     newname = newname != nullptr ? newname : "";
 
@@ -2526,11 +2516,15 @@ void tr_torrentSetFilePriorities(
     tr_file_index_t file_count,
     tr_priority_t priority)
 {
+    tr_return_if_fail(tr_isTorrent(tor));
+
     tor->set_file_priorities(files, file_count, priority);
 }
 
 bool tr_torrentHasMetadata(tr_torrent const* tor)
 {
+    tr_return_val_if_fail(tr_isTorrent(tor), {});
+
     return tor->has_metainfo();
 }
 
