@@ -1,4 +1,4 @@
-// This file Copyright © 2011-2023 Transmission authors and contributors.
+// This file Copyright © Transmission authors and contributors.
 // It may be used under the MIT (SPDX: MIT) license.
 // License text can be found in the licenses/ folder.
 
@@ -50,6 +50,7 @@ typedef NS_ENUM(NSInteger, FilterTypeTag) {
 
 - (void)awakeFromNib
 {
+    [super awakeFromNib];
     //localizations
     self.fNoFilterButton.title = NSLocalizedString(@"All", "Filter Bar -> filter button");
     self.fActiveFilterButton.title = NSLocalizedString(@"Active", "Filter Bar -> filter button");
@@ -140,11 +141,6 @@ typedef NS_ENUM(NSInteger, FilterTypeTag) {
     self.fSearchField.delegate = self;
 }
 
-- (void)dealloc
-{
-    [NSNotificationCenter.defaultCenter removeObserver:self];
-}
-
 - (void)setFilter:(id)sender
 {
     NSString* oldFilterType = [NSUserDefaults.standardUserDefaults stringForKey:@"Filter"];
@@ -323,24 +319,16 @@ typedef NS_ENUM(NSInteger, FilterTypeTag) {
     [NSNotificationCenter.defaultCenter postNotificationName:@"ApplyFilter" object:nil];
 }
 
-- (void)reset:(BOOL)updateUI
+- (void)reset
 {
     [NSUserDefaults.standardUserDefaults setInteger:kGroupFilterAllTag forKey:@"FilterGroup"];
 
-    if (updateUI)
-    {
-        [self updateGroupsButton];
+    [self updateGroupsButton];
 
-        [self setFilter:self.fNoFilterButton];
+    [self setFilter:self.fNoFilterButton];
 
-        self.fSearchField.stringValue = @"";
-        [self setSearchText:self.fSearchField];
-    }
-    else
-    {
-        [NSUserDefaults.standardUserDefaults setObject:FilterTypeNone forKey:@"Filter"];
-        [NSUserDefaults.standardUserDefaults removeObjectForKey:@"FilterSearchString"];
-    }
+    self.fSearchField.stringValue = @"";
+    [self setSearchText:self.fSearchField];
 }
 
 - (NSArray<NSString*>*)searchStrings
