@@ -1,4 +1,4 @@
-// This file Copyright © 2006-2023 Transmission authors and contributors.
+// This file Copyright © Transmission authors and contributors.
 // It may be used under the MIT (SPDX: MIT) license.
 // License text can be found in the licenses/ folder.
 
@@ -27,13 +27,13 @@ static CGFloat const kTabMinHeight = 250;
 
 static NSInteger const kInvalidTag = -99;
 
-typedef NS_ENUM(unsigned int, tabTag) {
-    TAB_GENERAL_TAG = 0,
-    TAB_ACTIVITY_TAG = 1,
-    TAB_TRACKERS_TAG = 2,
-    TAB_PEERS_TAG = 3,
-    TAB_FILE_TAG = 4,
-    TAB_OPTIONS_TAG = 5
+typedef NS_ENUM(NSUInteger, TabTag) {
+    TabTagGeneral = 0,
+    TabTagActivity = 1,
+    TabTagTrackers = 2,
+    TabTagPeers = 3,
+    TabTagFile = 4,
+    TabTagOptions = 5
 };
 
 @interface InfoWindowController ()
@@ -70,6 +70,7 @@ typedef NS_ENUM(unsigned int, tabTag) {
 
 - (void)awakeFromNib
 {
+    [super awakeFromNib];
     self.fNoneSelectedField.stringValue = NSLocalizedString(@"No Torrents Selected", "Inspector -> selected torrents");
 
     //window location and size
@@ -101,23 +102,29 @@ typedef NS_ENUM(unsigned int, tabTag) {
         [self.fTabs.cell setToolTip:toolTip forSegment:segment];
     };
     setImageAndToolTipForSegment(
-        [NSImage systemSymbol:@"info.circle" withFallback:@"InfoGeneral"],
+        [NSImage imageWithSystemSymbolName:@"info.circle" accessibilityDescription:nil],
         NSLocalizedString(@"General Info", "Inspector -> tab"),
-        TAB_GENERAL_TAG);
+        TabTagGeneral);
     setImageAndToolTipForSegment(
-        [NSImage systemSymbol:@"square.grid.3x3.fill.square" withFallback:@"InfoActivity"],
+        [NSImage imageWithSystemSymbolName:@"square.grid.3x3.fill.square" accessibilityDescription:nil],
         NSLocalizedString(@"Activity", "Inspector -> tab"),
-        TAB_ACTIVITY_TAG);
+        TabTagActivity);
     setImageAndToolTipForSegment(
-        [NSImage systemSymbol:@"antenna.radiowaves.left.and.right" withFallback:@"InfoTracker"],
+        [NSImage imageWithSystemSymbolName:@"antenna.radiowaves.left.and.right" accessibilityDescription:nil],
         NSLocalizedString(@"Trackers", "Inspector -> tab"),
-        TAB_TRACKERS_TAG);
-    setImageAndToolTipForSegment([NSImage systemSymbol:@"person.2" withFallback:@"InfoPeers"], NSLocalizedString(@"Peers", "Inspector -> tab"), TAB_PEERS_TAG);
-    setImageAndToolTipForSegment([NSImage systemSymbol:@"doc.on.doc" withFallback:@"InfoFiles"], NSLocalizedString(@"Files", "Inspector -> tab"), TAB_FILE_TAG);
+        TabTagTrackers);
     setImageAndToolTipForSegment(
-        [NSImage systemSymbol:@"gearshape" withFallback:@"InfoOptions"],
+        [NSImage imageWithSystemSymbolName:@"person.2" accessibilityDescription:nil],
+        NSLocalizedString(@"Peers", "Inspector -> tab"),
+        TabTagPeers);
+    setImageAndToolTipForSegment(
+        [NSImage imageWithSystemSymbolName:@"doc.on.doc" accessibilityDescription:nil],
+        NSLocalizedString(@"Files", "Inspector -> tab"),
+        TabTagFile);
+    setImageAndToolTipForSegment(
+        [NSImage imageWithSystemSymbolName:@"gearshape" accessibilityDescription:nil],
         NSLocalizedString(@"Options", "Inspector -> tab"),
-        TAB_OPTIONS_TAG);
+        TabTagOptions);
 
     //set selected tab
     self.fCurrentTabTag = kInvalidTag;
@@ -125,32 +132,32 @@ typedef NS_ENUM(unsigned int, tabTag) {
     NSInteger tag;
     if ([identifier isEqualToString:TabIdentifierInfo])
     {
-        tag = TAB_GENERAL_TAG;
+        tag = TabTagGeneral;
     }
     else if ([identifier isEqualToString:TabIdentifierActivity])
     {
-        tag = TAB_ACTIVITY_TAG;
+        tag = TabTagActivity;
     }
     else if ([identifier isEqualToString:TabIdentifierTracker])
     {
-        tag = TAB_TRACKERS_TAG;
+        tag = TabTagTrackers;
     }
     else if ([identifier isEqualToString:TabIdentifierPeers])
     {
-        tag = TAB_PEERS_TAG;
+        tag = TabTagPeers;
     }
     else if ([identifier isEqualToString:TabIdentifierFiles])
     {
-        tag = TAB_FILE_TAG;
+        tag = TabTagFile;
     }
     else if ([identifier isEqualToString:TabIdentifierOptions])
     {
-        tag = TAB_OPTIONS_TAG;
+        tag = TabTagOptions;
     }
     else //safety
     {
         [NSUserDefaults.standardUserDefaults setObject:TabIdentifierInfo forKey:@"InspectorSelected"];
-        tag = TAB_GENERAL_TAG;
+        tag = TabTagGeneral;
     }
 
     self.fTabs.selectedSegment = tag;
@@ -171,8 +178,6 @@ typedef NS_ENUM(unsigned int, tabTag) {
 
 - (void)dealloc
 {
-    [NSNotificationCenter.defaultCenter removeObserver:self];
-
     if ([_fViewController respondsToSelector:@selector(saveViewSize)])
     {
         [_fViewController saveViewSize];
@@ -203,6 +208,18 @@ typedef NS_ENUM(unsigned int, tabTag) {
     [self resetInfo];
 }
 
+- (void)removeTorrentsFromInfo:(NSArray<Torrent*>*)torrents
+{
+    if (self.fTorrents.count == 0 || torrents.count == 0)
+    {
+        return;
+    }
+
+    NSMutableArray<Torrent*>* remaining = [self.fTorrents mutableCopy];
+    [remaining removeObjectsInArray:torrents];
+    [self setInfoForTorrents:remaining];
+}
+
 - (NSRect)windowWillUseStandardFrame:(NSWindow*)window defaultFrame:(NSRect)defaultFrame
 {
     NSRect windowRect = window.frame;
@@ -212,7 +229,7 @@ typedef NS_ENUM(unsigned int, tabTag) {
 
 - (void)windowWillClose:(NSNotification*)notification
 {
-    if (self.fCurrentTabTag == TAB_FILE_TAG && ([QLPreviewPanel sharedPreviewPanelExists] && [QLPreviewPanel sharedPreviewPanel].visible))
+    if (self.fCurrentTabTag == TabTagFile && ([QLPreviewPanel sharedPreviewPanelExists] && [QLPreviewPanel sharedPreviewPanel].visible))
     {
         [[QLPreviewPanel sharedPreviewPanel] reloadData];
     }
@@ -255,7 +272,7 @@ typedef NS_ENUM(unsigned int, tabTag) {
     TabIdentifier identifier;
     switch (self.fCurrentTabTag)
     {
-    case TAB_GENERAL_TAG:
+    case TabTagGeneral:
         if (!self.fGeneralViewController)
         {
             self.fGeneralViewController = [[InfoGeneralViewController alloc] init];
@@ -265,7 +282,7 @@ typedef NS_ENUM(unsigned int, tabTag) {
         self.fViewController = self.fGeneralViewController;
         identifier = TabIdentifierInfo;
         break;
-    case TAB_ACTIVITY_TAG:
+    case TabTagActivity:
         if (!self.fActivityViewController)
         {
             self.fActivityViewController = [[InfoActivityViewController alloc] init];
@@ -275,7 +292,7 @@ typedef NS_ENUM(unsigned int, tabTag) {
         self.fViewController = self.fActivityViewController;
         identifier = TabIdentifierActivity;
         break;
-    case TAB_TRACKERS_TAG:
+    case TabTagTrackers:
         if (!self.fTrackersViewController)
         {
             self.fTrackersViewController = [[InfoTrackersViewController alloc] init];
@@ -285,7 +302,7 @@ typedef NS_ENUM(unsigned int, tabTag) {
         self.fViewController = self.fTrackersViewController;
         identifier = TabIdentifierTracker;
         break;
-    case TAB_PEERS_TAG:
+    case TabTagPeers:
         if (!self.fPeersViewController)
         {
             self.fPeersViewController = [[InfoPeersViewController alloc] init];
@@ -295,7 +312,7 @@ typedef NS_ENUM(unsigned int, tabTag) {
         self.fViewController = self.fPeersViewController;
         identifier = TabIdentifierPeers;
         break;
-    case TAB_FILE_TAG:
+    case TabTagFile:
         if (!self.fFileViewController)
         {
             self.fFileViewController = [[InfoFileViewController alloc] init];
@@ -305,7 +322,7 @@ typedef NS_ENUM(unsigned int, tabTag) {
         self.fViewController = self.fFileViewController;
         identifier = TabIdentifierFiles;
         break;
-    case TAB_OPTIONS_TAG:
+    case TabTagOptions:
         if (!self.fOptionsViewController)
         {
             self.fOptionsViewController = [[InfoOptionsViewController alloc] init];
@@ -325,7 +342,7 @@ typedef NS_ENUM(unsigned int, tabTag) {
     NSWindow* window = self.window;
 
     window.title = [NSString
-        stringWithFormat:@"%@ - %@", self.fViewController.title, NSLocalizedString(@"Torrent Inspector", "Inspector -> title")];
+        stringWithFormat:@"%@ — %@", self.fViewController.title, NSLocalizedString(@"Torrent Inspector", "Inspector -> title")];
 
     NSView* view = self.fViewController.view;
 
@@ -352,9 +369,9 @@ typedef NS_ENUM(unsigned int, tabTag) {
         viewRect = [self.fOptionsViewController viewRect];
     }
 
-    CGFloat const difference = NSHeight(viewRect) - oldHeight;
-    windowRect.origin.y -= difference;
-    windowRect.size.height += difference;
+    CGFloat const viewHeightDifference = NSHeight(viewRect) - oldHeight;
+    windowRect.origin.y -= viewHeightDifference;
+    windowRect.size.height += viewHeightDifference;
     windowRect.size.width = MAX(NSWidth(windowRect), minWindowWidth);
 
     if ([self.fViewController respondsToSelector:@selector(saveViewSize)]) //a little bit hacky, but avoids requiring an extra method
@@ -364,11 +381,11 @@ typedef NS_ENUM(unsigned int, tabTag) {
             CGFloat const screenHeight = NSHeight(window.screen.visibleFrame);
             if (NSHeight(windowRect) > screenHeight)
             {
-                CGFloat const difference = screenHeight - NSHeight(windowRect);
-                windowRect.origin.y -= difference;
-                windowRect.size.height += difference;
+                CGFloat const windowHeightDifference = screenHeight - NSHeight(windowRect);
+                windowRect.origin.y -= windowHeightDifference;
+                windowRect.size.height += windowHeightDifference;
 
-                viewRect.size.height += difference;
+                viewRect.size.height += windowHeightDifference;
             }
         }
 
@@ -415,7 +432,7 @@ typedef NS_ENUM(unsigned int, tabTag) {
         addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:[tabs]-0-[view]-0-|" options:0 metrics:nil
                                                                  views:@{ @"tabs" : self.fTabs, @"view" : view }]];
 
-    if ((self.fCurrentTabTag == TAB_FILE_TAG || oldTabTag == TAB_FILE_TAG) &&
+    if ((self.fCurrentTabTag == TabTagFile || oldTabTag == TabTagFile) &&
         ([QLPreviewPanel sharedPreviewPanelExists] && [QLPreviewPanel sharedPreviewPanel].visible))
     {
         [[QLPreviewPanel sharedPreviewPanel] reloadData];
@@ -482,7 +499,7 @@ typedef NS_ENUM(unsigned int, tabTag) {
 
 - (BOOL)canQuickLook
 {
-    if (self.fCurrentTabTag != TAB_FILE_TAG || !self.window.visible)
+    if (self.fCurrentTabTag != TabTagFile || !self.window.visible)
     {
         return NO;
     }
