@@ -73,8 +73,7 @@
 #ifdef __has_include
 #    if __has_include(<version>)
 #        include <version>
-#    endif
-#    if __has_include(<ciso646>)
+#    elif __has_include(<ciso646>)
 #        include <ciso646>
 #    endif
 #endif
