@@ -1,5 +1,5 @@
 ﻿///////////////////////////////////////////////////////////////////
-//  Copyright Christopher Kormanyos 2019 - 2024.                 //
+//  Copyright Christopher Kormanyos 2019 - 2025.                 //
 //  Distributed under the Boost Software License,                //
 //  Version 1.0. (See accompanying file LICENSE_1_0.txt          //
 //  or copy at http://www.boost.org/LICENSE_1_0.txt)             //
@@ -170,9 +170,9 @@
              std::enable_if_t<std::is_floating_point<FloatingPointType>::value> const* = nullptr>
     constexpr uintwide_t_backend(FloatingPointType f) : m_value(representation_type(static_cast<long double>(f))) { } // NOLINT(google-explicit-constructor,hicpp-explicit-conversions)
 
-    constexpr explicit uintwide_t_backend(const char* p_cstr) : m_value(p_cstr) { }
+    explicit constexpr uintwide_t_backend(const char* p_cstr) : m_value(p_cstr) { }
 
-    constexpr explicit uintwide_t_backend(const std::string& str) : m_value(str) { }
+    explicit constexpr uintwide_t_backend(const std::string& str) : m_value(str) { }
 
     //constexpr
     ~uintwide_t_backend() = default;
@@ -785,7 +785,7 @@
   (
           UnsignedIntegralType*                                                                 result,
     const uintwide_t_backend<MyWidth2, MyLimbType, MyAllocatorType>&                            val,
-          std::enable_if_t<(    (std::is_integral<UnsignedIntegralType>::value)
+          std::enable_if_t<(     std::is_integral<UnsignedIntegralType>::value
                             && (!std::is_signed  <UnsignedIntegralType>::value))>* p_nullparam = nullptr
   ) -> void
   {
