@@ -1,4 +1,4 @@
-/* @license This file Copyright © 2020-2023 Mnemosyne LLC.
+/* @license This file Copyright © Mnemosyne LLC.
    It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
    or any future license endorsed by Mnemosyne LLC.
    License text can be found in the licenses/ folder. */
@@ -7,24 +7,28 @@ export class ActionManager extends EventTarget {
   constructor() {
     super();
     this.actions = Object.seal({
+      'copy-name': {
+        enabled: true,
+        text: 'Copy name',
+      },
       'deselect-all': {
         enabled: false,
-        shortcut: 'Control+D',
+        shortcut: 'D',
         text: 'Deselect all',
       },
-      'move-bottom': { enabled: false, text: 'Move to the back of the queue' },
-      'move-down': { enabled: false, text: 'Move down in the queue' },
-      'move-top': { enabled: false, text: 'Move to the front of the queue' },
-      'move-up': { enabled: false, text: 'Move up in the queue' },
+      'move-bottom': { enabled: false, text: 'Bottom' },
+      'move-down': { enabled: false, text: 'Down' },
+      'move-top': { enabled: false, text: 'Top' },
+      'move-up': { enabled: false, text: 'Up' },
       'open-torrent': {
         enabled: true,
-        shortcut: 'Control+O',
+        shortcut: 'O',
         text: 'Open torrent…',
       },
       'pause-all-torrents': { enabled: false, text: 'Pause all' },
       'pause-selected-torrents': {
         enabled: false,
-        shortcut: 'Control+U',
+        shortcut: 'U',
         text: 'Pause',
       },
       'reannounce-selected-torrents': {
@@ -34,56 +38,58 @@ export class ActionManager extends EventTarget {
       'remove-selected-torrents': { enabled: false, text: 'Remove from list…' },
       'resume-selected-torrents': {
         enabled: false,
-        shortcut: 'Control+R',
+        shortcut: 'R',
         text: 'Resume',
       },
       'resume-selected-torrents-now': { enabled: false, text: 'Resume now' },
       'select-all': {
         enabled: false,
-        shortcut: 'Control+A',
+        shortcut: 'A',
         text: 'Select all',
       },
       'show-about-dialog': { enabled: true, text: 'About' },
       'show-inspector': {
         enabled: false,
-        shortcut: 'Control+I',
+        shortcut: 'I',
         text: 'Torrent Inspector',
       },
       'show-labels-dialog': {
         enabled: false,
+        shortcut: 'K',
         text: 'Edit Labels…',
       },
       'show-move-dialog': {
         enabled: false,
-        shortcut: 'Control+L',
+        shortcut: 'L',
         text: 'Set location…',
       },
       'show-overflow-menu': { enabled: true, text: 'More options…' },
       'show-preferences-dialog': {
         enabled: true,
-        shortcut: 'Control+P',
+        shortcut: 'P',
         text: 'Edit preferences',
       },
       'show-rename-dialog': {
         enabled: false,
-        shortcut: 'Control+N',
+        shortcut: 'N',
         text: 'Rename…',
       },
       'show-shortcuts-dialog': { enabled: true, text: 'Keyboard shortcuts' },
       'show-statistics-dialog': {
         enabled: true,
-        shortcut: 'Control+S',
+        shortcut: 'S',
         text: 'Statistics',
       },
       'start-all-torrents': { enabled: false, text: 'Start all' },
       'toggle-compact-rows': { enabled: true, text: 'Compact rows' },
+      'toggle-contrast': { enabled: true, text: 'High contrast UI' },
       'trash-selected-torrents': {
         enabled: false,
         text: 'Trash data and remove from list…',
       },
       'verify-selected-torrents': {
         enabled: false,
-        shortcut: 'Control+Shift+V',
+        shortcut: 'V',
         text: 'Verify local data',
       },
     });
@@ -111,7 +117,7 @@ export class ActionManager extends EventTarget {
     return new Map(
       Object.entries(this.actions)
         .filter(([, properties]) => properties.shortcut)
-        .map(([name, properties]) => [properties.shortcut, name])
+        .map(([name, properties]) => [properties.shortcut, name]),
     );
   }
 
@@ -145,7 +151,7 @@ export class ActionManager extends EventTarget {
     const selected_paused = selected.filter((tor) => tor.isStopped()).length;
     const selected_active = selected.length - selected_paused;
     const nonselected_paused = nonselected.filter((tor) =>
-      tor.isStopped()
+      tor.isStopped(),
     ).length;
     const nonselected_active = nonselected.length - nonselected_paused;
     const paused = selected_paused + nonselected_paused;
