@@ -1,9 +1,10 @@
-/* @license This file Copyright © 2020-2023 Mnemosyne LLC.
+/* @license This file Copyright © Mnemosyne LLC.
    It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
    or any future license endorsed by Mnemosyne LLC.
    License text can be found in the licenses/ folder. */
 
-import { createDialogContainer } from './utils.js';
+import { AlertDialog } from './alert-dialog.js';
+import { createDialogContainer, setTextContent } from './utils.js';
 
 export class RenameDialog extends EventTarget {
   constructor(controller, remote) {
@@ -24,22 +25,27 @@ export class RenameDialog extends EventTarget {
       return;
     }
 
+    const { handler } = this.controller;
     this.torrents = torrents;
     this.elements = RenameDialog._create();
     this.elements.dismiss.addEventListener('click', () => this._onDismiss());
     this.elements.confirm.addEventListener('click', () => this._onConfirm());
-    this.elements.entry.value = torrents[0].getName();
+    this.elements.entry.value =
+      handler === null ? torrents[0].getName() : handler.subtree.name;
     document.body.append(this.elements.root);
 
     this.elements.entry.focus();
   }
 
   close() {
+    const { handler } = this.controller;
+    if (handler) {
+      handler.classList.remove('selected');
+    }
     this.elements.root.remove();
 
     this.dispatchEvent(new Event('close'));
 
-    delete this.controller;
     delete this.remote;
     delete this.elements;
     delete this.torrents;
@@ -50,14 +56,48 @@ export class RenameDialog extends EventTarget {
   }
 
   _onConfirm() {
+    const { handler } = this.controller;
     const [tor] = this.torrents;
-    const old_name = tor.getName();
+    const file_path = handler ? handler.file_path : tor.getName();
     const new_name = this.elements.entry.value;
-    this.remote.renameTorrent([tor.getId()], old_name, new_name, (response) => {
-      if (response.result === 'success') {
-        tor.refresh(response.arguments);
-      }
-    });
+    this.remote.renameTorrent(
+      [tor.getId()],
+      file_path,
+      new_name,
+      (response) => {
+        if ('result' in response) {
+          const args = response.result;
+          if (handler) {
+            handler.subtree.name = args.name;
+            setTextContent(handler.name_container, args.name);
+            if (handler.subdir) {
+              const file = tor.getIndividualFile(file_path);
+              if (file) {
+                const dir = file.name.slice(
+                  0,
+                  Math.max(0, file.name.lastIndexOf('/') + 1),
+                );
+                file.name = `${dir}${args.name}`;
+              }
+            } else {
+              tor.refresh(args);
+            }
+          } else {
+            tor.refresh(args);
+          }
+        } else {
+          const error_obj = response.error;
+          const err_msg =
+            error_obj.data?.errorString ?? error_obj.message ?? '';
+          const connection_alert = new AlertDialog({
+            heading: `Error renaming "${file_path}"`,
+            message: `${err_msg} (${error_obj.code}`,
+          });
+          this.controller.setCurrentPopup(connection_alert);
+        }
+        delete this.controller;
+      },
+    );
 
     this.close();
   }
