@@ -1,4 +1,4 @@
-/* @license This file Copyright © 2020-2023 Charles Kerr, Dave Perrett, Malcolm Jarvis and Bruno Bierbaumer
+/* @license This file Copyright © Charles Kerr, Dave Perrett, Malcolm Jarvis and Bruno Bierbaumer
    It may be used under GPLv2 (SPDX: GPL-2.0-only).
    License text can be found in the licenses/ folder. */
 
@@ -23,7 +23,14 @@ import {
   TorrentRendererCompact,
   TorrentRendererFull,
 } from './torrent-row.js';
-import { debounce, deepEqual, setEnabled, setTextContent } from './utils.js';
+import {
+  newOpts,
+  icon,
+  debounce,
+  deepEqual,
+  setEnabled,
+  setTextContent,
+} from './utils.js';
 
 export class Transmission extends EventTarget {
   constructor(action_manager, notifications, prefs) {
@@ -31,28 +38,57 @@ export class Transmission extends EventTarget {
 
     // Initialize the helper classes
     this.action_manager = action_manager;
+    this.handler = null;
     this.notifications = notifications;
     this.prefs = prefs;
     this.remote = new Remote(this);
+    this.speed = {
+      down: document.querySelector('#speed-down'),
+      up: document.querySelector('#speed-up'),
+    };
+
+    for (const [selector, name] of [
+      ['#toolbar-open', 'open'],
+      ['#toolbar-delete', 'delete'],
+      ['#toolbar-start', 'start'],
+      ['#toolbar-pause', 'pause'],
+      ['#toolbar-inspector', 'inspector'],
+      ['#toolbar-overflow', 'overflow'],
+    ]) {
+      document
+        .querySelector(selector)
+        .prepend(icon[name](), document.createElement('BR'));
+    }
+
+    document.querySelector('.speed-container').append(icon.speedDown());
+    document
+      .querySelector('.speed-container + .speed-container')
+      .append(icon.speedUp());
 
     this.addEventListener('torrent-selection-changed', (event_) =>
-      this.action_manager.update(event_)
+      this.action_manager.update(event_),
     );
 
     // Initialize the implementation fields
     this.filterText = '';
     this._torrents = {};
     this._rows = [];
+    this.oldTrackers = [];
     this.dirtyTorrents = new Set();
 
+    this.changeStatus = false;
     this.refilterSoon = debounce(() => this._refilter(false));
     this.refilterAllSoon = debounce(() => this._refilter(true));
 
-    this.boundPopupCloseListener = this.popupCloseListener.bind(this);
-    this.dispatchSelectionChangedSoon = debounce(
-      () => this._dispatchSelectionChanged(),
-      200
-    );
+    this.pointer_device = Object.seal({
+      is_touch_device: 'ontouchstart' in globalThis,
+      long_press_callback: null,
+      x: 0,
+      y: 0,
+    });
+    this.popup = Array.from({ length: Transmission.max_popups }).fill(null);
+
+    this.busytyping = false;
 
     // listen to actions
     // TODO: consider adding a mutator listener here to see dynamic additions
@@ -67,14 +103,12 @@ export class Transmission extends EventTarget {
     document
       .querySelector('#filter-tracker')
       .addEventListener('change', (event_) => {
-        this.setFilterTracker(
-          event_.target.value === 'all' ? null : event_.target.value
-        );
+        this.setFilterTracker(event_.target.value);
       });
 
     this.action_manager.addEventListener('change', (event_) => {
       for (const element of document.querySelectorAll(
-        `[data-action="${event_.action}"]`
+        `[data-action="${event_.action}"]`,
       )) {
         setEnabled(element, event_.enabled);
       }
@@ -82,6 +116,16 @@ export class Transmission extends EventTarget {
 
     this.action_manager.addEventListener('click', (event_) => {
       switch (event_.action) {
+        case 'copy-name':
+          if (navigator.clipboard) {
+            navigator.clipboard.writeText(this.handler.subtree.name);
+          } else {
+            // navigator.clipboard requires HTTPS or localhost
+            // Emergency approach
+            prompt('Select all then copy', this.handler.subtree.name);
+          }
+          this.handler.classList.remove('selected');
+          break;
         case 'deselect-all':
           this._deselectAll();
           break;
@@ -125,27 +169,33 @@ export class Transmission extends EventTarget {
           this.setCurrentPopup(new AboutDialog(this.version_info));
           break;
         case 'show-inspector':
-          this.setCurrentPopup(new Inspector(this));
+          if (this.popup[0] instanceof Inspector) {
+            this.popup[0].close();
+          } else {
+            this.setCurrentPopup(new Inspector(this), 0);
+          }
           break;
         case 'show-move-dialog':
           this.setCurrentPopup(new MoveDialog(this, this.remote));
           break;
         case 'show-overflow-menu':
-          if (this.popup instanceof OverflowMenu) {
-            this.setCurrentPopup(null);
+          if (
+            this.popup[Transmission.default_popup_level] instanceof OverflowMenu
+          ) {
+            this.popup[Transmission.default_popup_level].close();
           } else {
             this.setCurrentPopup(
               new OverflowMenu(
                 this,
                 this.prefs,
                 this.remote,
-                this.action_manager
-              )
+                this.action_manager,
+              ),
             );
           }
           break;
         case 'show-preferences-dialog':
-          this.setCurrentPopup(new PrefsDialog(this, this.remote));
+          this.setCurrentPopup(new PrefsDialog(this, this.remote), 0);
           break;
         case 'show-shortcuts-dialog':
           this.setCurrentPopup(new ShortcutsDialog(this.action_manager));
@@ -179,38 +229,82 @@ export class Transmission extends EventTarget {
       }
     });
 
-    // listen to filter changes
     let e = document.querySelector('#filter-mode');
+    // Initialize filter options
+    newOpts(e, null, [['All', Prefs.FilterAll]]);
+    newOpts(e, 'status', [
+      ['Active', Prefs.FilterActive],
+      ['Downloading', Prefs.FilterDownloading],
+      ['Seeding', Prefs.FilterSeeding],
+      ['Paused', Prefs.FilterPaused],
+      ['Finished', Prefs.FilterFinished],
+      ['Error', Prefs.FilterError],
+    ]);
+    newOpts(e, 'list', [
+      ['Private torrents', Prefs.FilterPrivate],
+      ['Public torrents', Prefs.FilterPublic],
+    ]);
+
+    // listen to filter changes
     e.value = this.prefs.filter_mode;
     e.addEventListener('change', (event_) => {
       this.prefs.filter_mode = event_.target.value;
       this.refilterAllSoon();
     });
 
-    //if (!isMobileDevice) {
+    e = document.querySelector('#filter-tracker');
+    newOpts(e, null, [['All', Prefs.FilterAll]]);
+
+    const s = document.querySelector('#torrent-search');
+    e = document.querySelector('#reset');
+    e.addEventListener('click', () => {
+      s.value = '';
+      this._setFilterText(s.value);
+      this.refilterAllSoon();
+    });
+
+    if (s.value.trim()) {
+      this.filterText = s.value;
+      e.style.display = 'block';
+      this.refilterAllSoon();
+    }
+
+    e = document.querySelector('#turtle');
+    e.addEventListener('click', (event_) => {
+      this.remote.savePrefs({
+        [RPC._TurtleState]:
+          !event_.target.classList.contains('alt-speed-enabled'),
+      });
+    });
+
     document.addEventListener('keydown', this._keyDown.bind(this));
     document.addEventListener('keyup', this._keyUp.bind(this));
     e = document.querySelector('#torrent-container');
-    e.addEventListener('click', () => {
-      if (this.popup && this.popup.name !== 'inspector') {
+    e.addEventListener('click', (e_) => {
+      if (this.popup[Transmission.default_popup_level]) {
         this.setCurrentPopup(null);
-      } else {
+      }
+      if (e_.target === e_.currentTarget) {
         this._deselectAll();
       }
     });
+    e.addEventListener('dblclick', () => {
+      if (!this.popup[0] || this.popup[0].name !== 'inspector') {
+        this.action_manager.click('show-inspector');
+      }
+    });
     e.addEventListener('dragenter', Transmission._dragenter);
     e.addEventListener('dragover', Transmission._dragenter);
     e.addEventListener('drop', this._drop.bind(this));
     this._setupSearchBox();
-    //}
 
     this.elements = {
       torrent_list: document.querySelector('#torrent-list'),
     };
 
-    this.elements.torrent_list.addEventListener('contextmenu', (event_) => {
-      // ensure the clicked row is selected
-      let row_element = event.target;
+    const right_click = (event_) => {
+      // if not already, highlight the torrent
+      let row_element = event_.target;
       while (row_element && !row_element.classList.contains('torrent')) {
         row_element = row_element.parentNode;
       }
@@ -219,23 +313,16 @@ export class Transmission extends EventTarget {
         this._setSelectedRow(row);
       }
 
-      const popup = new ContextMenu(this.action_manager);
-      this.setCurrentPopup(popup);
+      if (this.handler) {
+        this.handler.classList.remove('selected');
+        this.handler = null;
+      }
 
-      const boundingElement = document.querySelector('#torrent-container');
-      const bounds = boundingElement.getBoundingClientRect();
-      const x = Math.min(
-        event_.x,
-        bounds.x + bounds.width - popup.root.clientWidth
-      );
-      const y = Math.min(
-        event_.y,
-        bounds.y + bounds.height - popup.root.clientHeight
-      );
-      popup.root.style.left = `${x > 0 ? x : 0}px`;
-      popup.root.style.top = `${y > 0 ? y : 0}px`;
+      this.context_menu('#torrent-container');
       event_.preventDefault();
-    });
+    };
+
+    this.pointer_event(this.elements.torrent_list, right_click);
 
     // Get preferences & torrents from the daemon
     this.loadDaemonPrefs();
@@ -246,7 +333,7 @@ export class Transmission extends EventTarget {
     // this.updateButtonsSoon();
 
     this.prefs.addEventListener('change', ({ key, value }) =>
-      this._onPrefChanged(key, value)
+      this._onPrefChanged(key, value),
     );
     for (const [key, value] of this.prefs.entries()) {
       this._onPrefChanged(key, value);
@@ -255,21 +342,21 @@ export class Transmission extends EventTarget {
 
   _openTorrentFromUrl() {
     setTimeout(() => {
-      const addTorrent = new URLSearchParams(window.location.search).get(
-        'addtorrent'
+      const addTorrent = new URLSearchParams(globalThis.location.search).get(
+        'addtorrent',
       );
       if (addTorrent) {
         this.setCurrentPopup(new OpenDialog(this, this.remote, addTorrent));
-        const newUrl = new URL(window.location);
+        const newUrl = new URL(globalThis.location);
         newUrl.search = '';
-        window.history.pushState('', '', newUrl.toString());
+        globalThis.history.pushState('', '', newUrl.toString());
       }
     }, 0);
   }
 
   loadDaemonPrefs() {
     this.remote.loadDaemonPrefs((data) => {
-      this.session_properties = data.arguments;
+      this.session_properties = data.result;
       this._openTorrentFromUrl();
     });
   }
@@ -297,7 +384,11 @@ export class Transmission extends EventTarget {
     e.classList.add(blur_token);
     e.addEventListener('blur', () => e.classList.add(blur_token));
     e.addEventListener('focus', () => e.classList.remove(blur_token));
-    e.addEventListener('keyup', () => this._setFilterText(e.value));
+    e.addEventListener('input', () => {
+      if (e.value.trim() !== this.filterText) {
+        this._setFilterText(e.value);
+      }
+    });
   }
 
   _onPrefChanged(key, value) {
@@ -310,6 +401,13 @@ export class Transmission extends EventTarget {
         this.refilterAllSoon();
         break;
       }
+      case Prefs.ContrastMode: {
+        // Add custom class to the body/html element to get the appropriate contrast color scheme
+        document.body.classList.remove('contrast-more', 'contrast-less');
+        document.body.classList.add(`contrast-${value}`);
+        // this.refilterAllSoon();
+        break;
+      }
 
       case Prefs.FilterMode:
       case Prefs.SortDirection:
@@ -320,7 +418,8 @@ export class Transmission extends EventTarget {
       case Prefs.RefreshRate: {
         clearInterval(this.refreshTorrentsInterval);
         const callback = this.refreshTorrents.bind(this);
-        const msec = Math.max(2, this.prefs.refresh_rate_sec) * 1000;
+        const pref = this.prefs.refresh_rate_sec;
+        const msec = pref > 0 ? pref * 1000 : 1000;
         this.refreshTorrentsInterval = setInterval(callback, msec);
         break;
       }
@@ -331,8 +430,87 @@ export class Transmission extends EventTarget {
     }
   }
 
+  context_menu(container_id, menu_items) {
+    // open context menu
+    const popup = new ContextMenu(this, menu_items);
+    this.setCurrentPopup(popup);
+
+    const bounds = document.querySelector(container_id).getBoundingClientRect();
+    const x = Math.min(
+      this.pointer_device.x,
+      bounds.right + globalThis.scrollX - popup.root.clientWidth,
+    );
+    const y = Math.min(
+      this.pointer_device.y,
+      bounds.bottom + globalThis.scrollY - popup.root.clientHeight,
+    );
+    popup.root.style.left = `${Math.max(x, 0)}px`;
+    popup.root.style.top = `${Math.max(y, 0)}px`;
+  }
+
+  pointer_event(e_, right_click) {
+    if (this.pointer_device.is_touch_device) {
+      const touch = this.pointer_device;
+      e_.addEventListener('touchstart', (event_) => {
+        touch.x = event_.touches[0].pageX;
+        touch.y = event_.touches[0].pageY;
+
+        if (touch.long_press_callback) {
+          clearTimeout(touch.long_press_callback);
+          touch.long_press_callback = null;
+        } else {
+          touch.long_press_callback = setTimeout(() => {
+            if (event_.touches.length === 1) {
+              right_click(event_);
+            }
+          }, 500);
+        }
+      });
+      e_.addEventListener('touchend', () => {
+        clearTimeout(touch.long_press_callback);
+        touch.long_press_callback = null;
+        setTimeout(() => {
+          const popup = this.popup[Transmission.default_popup_level];
+          if (popup) {
+            popup.root.style.pointerEvents = 'auto';
+          }
+        }, 1);
+      });
+      e_.addEventListener('touchmove', (event_) => {
+        touch.x = event_.touches[0].pageX;
+        touch.y = event_.touches[0].pageY;
+
+        clearTimeout(touch.long_press_callback);
+        touch.long_press_callback = null;
+      });
+      e_.addEventListener('contextmenu', (event_) => {
+        event_.preventDefault();
+      });
+    } else {
+      e_.addEventListener('mousemove', (event_) => {
+        this.pointer_device.x = event_.pageX;
+        this.pointer_device.y = event_.pageY;
+      });
+      e_.addEventListener('contextmenu', (event_) => {
+        right_click(event_);
+        const popup = this.popup[Transmission.default_popup_level];
+        if (popup) {
+          popup.root.style.pointerEvents = 'auto';
+        }
+      });
+    }
+  }
+
   /// UTILITIES
 
+  static get max_popups() {
+    return 2;
+  }
+
+  static get default_popup_level() {
+    return Transmission.max_popups - 1;
+  }
+
   _getAllTorrents() {
     return Object.values(this._torrents);
   }
@@ -343,8 +521,8 @@ export class Transmission extends EventTarget {
 
   seedRatioLimit() {
     const p = this.session_properties;
-    if (p && p.seedRatioLimited) {
-      return p.seedRatioLimit;
+    if (p && p.seed_ratio_limited) {
+      return p.seed_ratio_limit;
     }
     return -1;
   }
@@ -368,37 +546,37 @@ export class Transmission extends EventTarget {
     for (const e of this.elements.torrent_list.children) {
       e.classList.toggle('selected', e === e_sel);
     }
-    this.dispatchSelectionChangedSoon();
+    this._dispatchSelectionChanged();
   }
 
   _selectRow(row) {
     row.getElement().classList.add('selected');
-    this.dispatchSelectionChangedSoon();
+    this._dispatchSelectionChanged();
   }
 
   _deselectRow(row) {
     row.getElement().classList.remove('selected');
-    this.dispatchSelectionChangedSoon();
+    this._dispatchSelectionChanged();
   }
 
   _selectAll() {
     for (const e of this.elements.torrent_list.children) {
       e.classList.add('selected');
     }
-    this.dispatchSelectionChangedSoon();
+    this._dispatchSelectionChanged();
   }
 
   _deselectAll() {
     for (const e of this.elements.torrent_list.children) {
       e.classList.remove('selected');
     }
-    this.dispatchSelectionChangedSoon();
+    this._dispatchSelectionChanged();
     delete this._last_torrent_clicked;
   }
 
   _indexOfLastTorrent() {
     return this._rows.findIndex(
-      (row) => row.getTorrentId() === this._last_torrent_clicked
+      (row) => row.getTorrentId() === this._last_torrent_clicked,
     );
   }
 
@@ -418,7 +596,7 @@ export class Transmission extends EventTarget {
       }
     }
 
-    this.dispatchSelectionChangedSoon();
+    this._dispatchSelectionChanged();
   }
 
   _dispatchSelectionChanged() {
@@ -451,7 +629,7 @@ export class Transmission extends EventTarget {
     if (event_.metaKey) {
       a.push('Meta');
     }
-    if (event_.shitKey) {
+    if (event_.shiftKey) {
       a.push('Shift');
     }
     a.push(event_.key.length === 1 ? event_.key.toUpperCase() : event_.key);
@@ -463,7 +641,7 @@ export class Transmission extends EventTarget {
     const { ctrlKey, keyCode, metaKey, shiftKey, target } = event_;
 
     // look for a shortcut
-    const is_input_focused = target.matches('input');
+    const is_input_focused = ['INPUT', 'TEXTAREA'].includes(target.tagName);
     if (!is_input_focused) {
       const shortcut = Transmission._createKeyShortcutFromKeyboardEvent(event_);
       const action = this.action_manager.getActionForShortcut(shortcut);
@@ -475,8 +653,8 @@ export class Transmission extends EventTarget {
     }
 
     const esc_key = keyCode === 27; // esc key pressed
-    if (esc_key && this.popup) {
-      this.setCurrentPopup(null);
+    if (esc_key && this.popup.some(Boolean)) {
+      this.setCurrentPopup(null, 0);
       event_.preventDefault();
       return;
     }
@@ -549,7 +727,10 @@ export class Transmission extends EventTarget {
   static _dragenter(event_) {
     if (event_.dataTransfer && event_.dataTransfer.types) {
       const copy_types = new Set(['text/uri-list', 'text/plain']);
-      if (event_.dataTransfer.types.some((type) => copy_types.has(type))) {
+      if (
+        event_.dataTransfer.types.some((type) => copy_types.has(type)) ||
+        event_.dataTransfer.types.includes('Files')
+      ) {
         event_.stopPropagation();
         event_.preventDefault();
         event_.dataTransfer.dropEffect = 'copy';
@@ -564,14 +745,14 @@ export class Transmission extends EventTarget {
   static _isValidURL(string) {
     try {
       const url = new URL(string);
-      return url ? true : false;
+      return Boolean(url);
     } catch {
       return false;
     }
   }
 
   shouldAddedTorrentsStart() {
-    return this.session_properties['start-added-torrents'];
+    return this.session_properties.start_added_torrents;
   }
 
   _drop(event_) {
@@ -581,9 +762,9 @@ export class Transmission extends EventTarget {
       return true;
     }
 
-    const type = event_.data.Transfer.types
-      .filter((t) => ['text/uri-list', 'text/plain'].contains(t))
-      .pop();
+    const type = event_.dataTransfer.types.findLast((t) =>
+      ['text/uri-list', 'text/plain'].includes(t),
+    );
     for (const uri of event_.dataTransfer
       .getData(type)
       .split('\n')
@@ -592,6 +773,11 @@ export class Transmission extends EventTarget {
       this.remote.addTorrentByUrl(uri, paused);
     }
 
+    const { files } = event_.dataTransfer;
+
+    if (files.length > 0) {
+      this.setCurrentPopup(new OpenDialog(this, this.remote, '', files));
+    }
     event_.preventDefault();
     return false;
   }
@@ -608,18 +794,30 @@ export class Transmission extends EventTarget {
         const msec = 8000;
         this.sessionInterval = setInterval(
           this.loadDaemonPrefs.bind(this),
-          msec
+          msec,
         );
       }
     }
   }
 
   _setFilterText(search) {
-    this.filterText = search ? search.trim() : null;
-    this.refilterAllSoon();
+    clearTimeout(this.busytyping);
+    this.busytyping = setTimeout(
+      () => {
+        this.busytyping = false;
+        this.filterText = search.trim();
+        this.refilterAllSoon();
+      },
+      search ? 250 : 0,
+    );
   }
 
   _onTorrentChanged(event_) {
+    if (this.changeStatus) {
+      this._dispatchSelectionChanged();
+      this.changeStatus = false;
+    }
+
     // update our dirty fields
     const tor = event_.currentTarget;
     this.dirtyTorrents.add(tor.getId());
@@ -696,7 +894,7 @@ TODO: fix this when notifications get fixed
 
   refreshTorrents() {
     const fields = ['id', ...Torrent.Fields.Stats];
-    this.updateTorrents('recently-active', fields);
+    this.updateTorrents('recently_active', fields);
   }
 
   _initializeTorrents() {
@@ -708,34 +906,18 @@ TODO: fix this when notifications get fixed
     const meta_key = event_.metaKey || event_.ctrlKey,
       { row } = event_.currentTarget;
 
-    if (this.popup && this.popup.name !== 'inspector') {
+    if (this.popup[Transmission.default_popup_level]) {
       this.setCurrentPopup(null);
-      return;
-    }
-
-    // handle the per-row pause/resume button
-    if (event_.target.classList.contains('torrent-pauseresume-button')) {
-      switch (event_.target.dataset.action) {
-        case 'pause':
-          this._stopTorrents([row.getTorrent()]);
-          break;
-        case 'resume':
-          this._startTorrents([row.getTorrent()]);
-          break;
-        default:
-          break;
-      }
     }
 
     // Prevents click carrying to parent element
     // which deselects all on click
     event_.stopPropagation();
 
-    // TODO: long-click should raise inspector
     if (event_.shiftKey) {
       this._selectRange(row);
       // Need to deselect any selected text
-      window.focus();
+      globalThis.focus();
 
       // Apple-Click, not selected
     } else if (!row.isSelected() && meta_key) {
@@ -771,7 +953,7 @@ TODO: fix this when notifications get fixed
     const torrents = this.getSelectedTorrents();
     if (torrents.length > 0) {
       this.setCurrentPopup(
-        new RemoveDialog({ remote: this.remote, torrents, trash })
+        new RemoveDialog({ remote: this.remote, torrents, trash }),
       );
     }
   }
@@ -781,18 +963,19 @@ TODO: fix this when notifications get fixed
   }
 
   _startTorrents(torrents, force) {
+    this.changeStatus = true;
     this.remote.startTorrents(
       Transmission._getTorrentIds(torrents),
       force,
       this.refreshTorrents,
-      this
+      this,
     );
   }
   _verifyTorrents(torrents) {
     this.remote.verifyTorrents(
       Transmission._getTorrentIds(torrents),
       this.refreshTorrents,
-      this
+      this,
     );
   }
 
@@ -800,15 +983,20 @@ TODO: fix this when notifications get fixed
     this.remote.reannounceTorrents(
       Transmission._getTorrentIds(torrents),
       this.refreshTorrents,
-      this
+      this,
     );
   }
 
   _stopTorrents(torrents) {
+    this.changeStatus = true;
     this.remote.stopTorrents(
       Transmission._getTorrentIds(torrents),
-      this.refreshTorrents,
-      this
+      () => {
+        setTimeout(() => {
+          this.refreshTorrents();
+        }, 500);
+      },
+      this,
     );
   }
   changeFileCommand(torrentId, rowIndices, command) {
@@ -820,41 +1008,41 @@ TODO: fix this when notifications get fixed
     this.remote.moveTorrentsToTop(
       this._getSelectedTorrentIds(),
       this.refreshTorrents,
-      this
+      this,
     );
   }
   _moveUp() {
     this.remote.moveTorrentsUp(
       this._getSelectedTorrentIds(),
       this.refreshTorrents,
-      this
+      this,
     );
   }
   _moveDown() {
     this.remote.moveTorrentsDown(
       this._getSelectedTorrentIds(),
       this.refreshTorrents,
-      this
+      this,
     );
   }
   _moveBottom() {
     this.remote.moveTorrentsToBottom(
       this._getSelectedTorrentIds(),
       this.refreshTorrents,
-      this
+      this,
     );
   }
 
   ///
 
   _updateGuiFromSession(o) {
-    const [, version, checksum] = o.version.match(/(.*)\s\(([\da-f]+)\)/);
+    const [, version, checksum] = o.version.match(/^(.*)\s\(([\da-f]+)\)/);
     this.version_info = {
       checksum,
       version,
     };
 
-    const element = document.querySelector('#toolbar-overflow');
+    const element = document.querySelector('#turtle');
     element.classList.toggle('alt-speed-enabled', o[RPC._TurtleState]);
   }
 
@@ -864,16 +1052,16 @@ TODO: fix this when notifications get fixed
 
     const u = torrents.reduce(
       (accumulator, tor) => accumulator + tor.getUploadSpeed(),
-      0
+      0,
     );
     const d = torrents.reduce(
       (accumulator, tor) => accumulator + tor.getDownloadSpeed(),
-      0
+      0,
     );
     const string = fmt.countString('Transfer', 'Transfers', this._rows.length);
 
-    setTextContent(document.querySelector('#speed-up-label'), fmt.speedBps(u));
-    setTextContent(document.querySelector('#speed-dn-label'), fmt.speedBps(d));
+    setTextContent(this.speed.down, fmt.speedBps(d));
+    setTextContent(this.speed.up, fmt.speedBps(u));
     setTextContent(document.querySelector('#filter-count'), string);
   }
 
@@ -887,24 +1075,29 @@ TODO: fix this when notifications get fixed
 
   _updateFilterSelect() {
     const trackers = this._getTrackerCounts();
-    const sitenames = Object.keys(trackers).sort();
+    const sitenames = Object.keys(trackers).toSorted();
 
-    // build the new html
-    let string = '';
-    string += this.filterTracker
-      ? '<option value="all">All</option>'
-      : '<option value="all" selected="selected">All</option>';
-    for (const sitename of sitenames) {
-      string += `<option value="${sitename}"`;
-      if (sitename === this.filterTracker) {
-        string += ' selected="selected"';
+    // Update select box only when list of trackers has changed
+    if (
+      sitenames.length !== this.oldTrackers.length ||
+      sitenames.some((ele, idx) => ele !== this.oldTrackers[idx])
+    ) {
+      this.oldTrackers = sitenames;
+
+      const a = [
+        ['All', Prefs.FilterAll, !this.filterTracker],
+        ...sitenames.map((sitename) => [
+          Transmission._displayName(sitename),
+          sitename,
+          sitename === this.filterTracker,
+        ]),
+      ];
+
+      const e = document.querySelector('#filter-tracker');
+      while (e.firstChild) {
+        e.lastChild.remove();
       }
-      string += `>${Transmission._displayName(sitename)}</option>`;
-    }
-
-    if (!this.filterTrackersStr || this.filterTrackersStr !== string) {
-      this.filterTrackersStr = string;
-      document.querySelector('#filter-tracker').innerHTML = string;
+      newOpts(e, null, a);
     }
   }
 
@@ -919,7 +1112,7 @@ TODO: fix this when notifications get fixed
     Torrent.sortTorrents(
       torrents,
       this.prefs.sort_mode,
-      this.prefs.sort_direction
+      this.prefs.sort_direction,
     );
     for (const [index, tor] of torrents.entries()) {
       rows[index] = id2row[tor.getId()];
@@ -934,6 +1127,9 @@ TODO: fix this when notifications get fixed
 
     let filter_text = null;
     let labels = null;
+    // TODO: This regex is wrong and is about to be removed in https://github.com/transmission/transmission/pull/7008,
+    // so it is left alone for now.
+    // eslint-disable-next-line sonarjs/slow-regex
     const m = /^labels:([\w,-\s]*)(.*)$/.exec(this.filterText);
     if (m) {
       filter_text = m[2].trim();
@@ -947,22 +1143,22 @@ TODO: fix this when notifications get fixed
     const countSelectedRows = () =>
       [...list.children].reduce(
         (n, e) => (n + e.classList.contains('selected') ? 1 : 0),
-        0
+        0,
       );
     const old_row_count = countRows();
     const old_sel_count = countSelectedRows();
 
     this._updateFilterSelect();
 
-    clearTimeout(this.refilterTimer);
-    delete this.refilterTimer;
-
     if (rebuildEverything) {
       while (list.firstChild) {
         list.firstChild.remove();
       }
       this._rows = [];
       this.dirtyTorrents = new Set(Object.keys(this._torrents));
+
+      document.querySelector('#reset').style.display =
+        this.filterText.length > 0 ? 'block' : 'none';
     }
 
     // rows that overlap with dirtyTorrents need to be refiltered.
@@ -1004,9 +1200,6 @@ TODO: fix this when notifications get fixed
         e.row = row;
         dirty_rows.push(row);
         e.addEventListener('click', this._onRowClicked.bind(this));
-        e.addEventListener('dblclick', () =>
-          this.action_manager.click('show-inspector')
-        );
       }
     }
 
@@ -1032,7 +1225,7 @@ TODO: fix this when notifications get fixed
           clean_rows[ci].getTorrent(),
           dirty_rows[di].getTorrent(),
           sort_mode,
-          sort_direction
+          sort_direction,
         );
         push_clean = c < 0;
       }
@@ -1058,19 +1251,12 @@ TODO: fix this when notifications get fixed
     this._rows = rows;
     this.dirtyTorrents.clear();
 
-    // set the odd/even property
-    for (const [index, e] of rows.map((row) => row.getElement()).entries()) {
-      const even = index % 2 === 0;
-      e.classList.toggle('even', even);
-      e.classList.toggle('odd', !even);
-    }
-
     this._updateStatusbar();
     if (
       old_sel_count !== countSelectedRows() ||
       old_row_count !== countRows()
     ) {
-      this.dispatchSelectionChangedSoon();
+      this._dispatchSelectionChanged();
     }
   }
 
@@ -1078,7 +1264,7 @@ TODO: fix this when notifications get fixed
     const e = document.querySelector('#filter-tracker');
     e.value = sitename;
 
-    this.filterTracker = sitename;
+    this.filterTracker = sitename === Prefs.FilterAll ? '' : sitename;
     this.refilterAllSoon();
   }
 
@@ -1097,23 +1283,25 @@ TODO: fix this when notifications get fixed
 
   ///
 
-  popupCloseListener(event_) {
-    if (event_.target !== this.popup) {
-      throw new Error(event_);
-    }
-    this.popup.removeEventListener('close', this.boundPopupCloseListener);
-    delete this.popup;
-  }
-
-  setCurrentPopup(popup) {
-    if (this.popup) {
-      this.popup.close();
+  setCurrentPopup(popup, level = Transmission.default_popup_level) {
+    for (let index = level; index < Transmission.max_popups; index++) {
+      if (this.popup[index]) {
+        this.popup[index].close();
+      }
     }
 
-    this.popup = popup;
+    this.popup[level] = popup;
 
-    if (this.popup) {
-      this.popup.addEventListener('close', this.boundPopupCloseListener);
+    if (this.popup[level]) {
+      const listener = () => {
+        if (this.popup[level]) {
+          this.popup[level].removeEventListener('close', listener);
+          this.popup[level] = null;
+        }
+      };
+      this.popup[level].addEventListener('close', listener);
+    } else if (this.handler) {
+      this.handler.classList.remove('selected');
     }
   }
 }
